/*******************************************************************
 * Modul...: usercachedb_db.cc                                     *
 * Zweck...: Implenemtierung der Klasse Usercachdb_db              *
 * Author..: Rudolf Weber Informatikberatung Ravensburg            *
 * Copyright 1997 GNU_Copyleft                                     *
 *******************************************************************/
#include<stdio.h>
#include<assert.h>
#include<errno.h>
#include"logmsg.h"
#include"usercachedb_db.h"
#include"config.h"
#include<unistd.h>
#include<sys/stat.h>

DbEnv Usercachedb_db::dbenv;
DbEnv *Usercachedb_db::dbenvp=NULL;
extern char *progname;
extern int debugflags;

DbEnv *Usercachedb_db::init_dbenv()
{
  static char errpfx[64];
  char *config[2];
  struct stat stbuf;
 
  snprintf(errpfx,64,"%s db:",progname);
  dbenv.set_errpfx(errpfx);
  
  /* Schaue ob DB_DATA_DIR existiert und versuche es ggf. anzulegen */
  if(stat(DATABASE_HOME "/files",&stbuf)<0) 
  {
    char puffer[256];
    if(mkdir(DATABASE_HOME "/files",0770)<0)
    {
      snprintf(puffer,256,"chdir %s : %s ",DATABASE_HOME "/files",strerror(errno));
      throw DbException(puffer);
    }
  }
  else
  {
    char puffer[256];
    if(!S_ISDIR(stbuf.st_mode))
    {
      snprintf(puffer,256,"%s ist kein Verzeichnis",DATABASE_HOME "/files");
      throw DbException(puffer);
    }
  }
    
  config[0] = "DB_DATA_DIR " DATABASE_HOME "/files";
  config[1] = NULL;

  if((errno=dbenv.appinit(DATABASE_HOME,config, DB_CREATE|DB_INIT_LOCK|DB_INIT_MPOOL))!=0)
  {
    logmsg(LOG_ERR,"%s:db_appinit : %s",progname,strerror(errno));
    //exit(1);
  }
  return &dbenv;
}

int Usercachedb_db::init(char *datname,int flags)
{
  int db_flags=0;
  assert(!((flags&UDB_WRITE)==0 && (flags&UDB_CREATE)==UDB_CREATE));
  if((flags&UDB_WRITE)==0) db_flags|=DB_RDONLY;
  if((flags&UDB_CREATE)==UDB_CREATE) db_flags|=DB_CREATE;
  if((flags&UDB_TRUNC)==UDB_TRUNC)  db_flags|=DB_TRUNCATE;
  
  if(dbenvp == NULL) dbenvp=init_dbenv();
  Db::open(datname,DB_HASH,flags,0640,dbenvp,(DbInfo *)0,&db);
  isopen=1;
}

int Usercachedb_db::close()
{
  // Leider macht das die berkleydb nicht selber 
  // notwendig: close sowohl im desdruktor als auch expizit aufrufen
  if(isopen)
  {
    isopen=0;
    if(debugflags) cerr << "Usercachedb_db::close(): schliesse " << endl;
    return db->close(0);
  }
  else
  {
    if(debugflags) cerr << "Usercachedb_db::close(): war schon zu" << endl;
    return 0;
  }
}

int Usercachedb_db::loesche(const char *name)
{
  int ret;
  Dbt key(name,strlen(name));
  ret = db->del(0,&key,0);
  return ret;
}

int Usercachedb_db::lese(const char *name,int nlen,Cacherecverw *urv)
{
  int ret;
  Dbt key(name,nlen);
  Dbt data;
  data.set_flags(DB_DBT_USERMEM);
  data.set_data(urv->data());
  data.set_ulen(urv->bufgross());

  ret = db->get(0,&key,&data,0);
  if(ret == 0) return 0;
  else if(ret==DB_NOTFOUND) return -1;
  else
  {
    throw DbException("Usercachedb::lese:",ret);
    return ret;
  }
}

int Usercachedb_db::schreibe(const char *name,Cacherecverw *urv)
{
  Dbt key(name,strlen(name));
  Dbt data(urv->data(),urv->totallen());

  db->put(0,&key,&data,0);
}

#ifdef USERQUERY
#include"config.h"
Attrdict dict;
Usercachedb_db udb;
char *progname;
int debugflags=0;
char buffer[4048];

main(int argc,char **argv)
{
    char name[256];
    Cacherecverw urv(buffer,4048);
    logmsginit(LOGMSG_STDERR,"userquery",0,0);
    if(argc<2)
    {
      cerr << "Aufruf: userquery <dbname>\n";
      exit(1);
    }
    try
    {
      progname=argv[0];
      dict.init(RADDICT);
      udb.init(argv[1],UDB_READ);
    }
    catch(DbException &dbe)
    {
      logmsg(LOG_ERR,dbe.what());
      exit(1);
    }
    
    cout << "Username ?";
    cin >> name;
    while(!cin.eof())
    {
      try
      {
	if(udb.lese(name,strlen(name),&urv)>=0)
	{
	  urv.print(name,cout);
	}
	else
	{
	  cout << "Nutzer " << name << " nicht gefunden ! \n";
	}
      }
      catch(DbException &dbe)
      {
	logmsg(LOG_ERR,dbe.what());
      }

      cout << "Username ?";
      cin >> name;      
    }
}
#endif




