/*************************************************************************
 * Klasse...: Userscache                                                 *
 * Zweck....: Verwaltet Radius-Userdatei/cache eintrag                   *
 *            Die Userdateieintrge werden im hnlichen Format wie die   *
 *            Nachrichtenpakete gespeichert.                             *
 * Copyright 1997 GNU_Copyleft                                           *
 *************************************************************************/  
#ifndef USERSCACHE_H
#define USERSCACHE_H
#include"authattr.h"

struct CRDienst
{
  UINT4     dienst;   // Dienste /Userservice
                      // ( User-service << 16 | Framed-Protokol)
  time_t    czeit;    // Zeit des Cachings
  u_int16_t offset;   // offset der Nutzdaten vom beginn des Satzes
  u_int16_t lang;     // Lnge der Nutzdaten
};

const UINT4 HDIENSTMASK=0xffff;
inline UINT4 hauptdienst(UINT4 d) { return d & HDIENSTMASK; } 
inline UINT4 dienstattr(UINT4 d)  { return d >> 16; }
const char *parsedienst(UINT4 &ret,char *&p,Attreintrag *service,Attreintrag *proto); // in usersparse.c implementiert
inline mergeservice(UINT4 &d,UINT4 s) 
{
  d &= ~HDIENSTMASK; // Hauptdienst = 0;
  d |= (s & HDIENSTMASK);
}
inline mergeproto(UINT4 &d,UINT4 p)
{
  d &= HDIENSTMASK;  // Potokoll = 0;
  d |= (p << 16);
}

// Layout:
// ----------------------------------------------------------
// | zeit | ng | CRDienst |daten | CRDienst | Daten | .....
// ----------------------------------------------------------

struct Cacherecord
{
  u_int32_t ng;       // Zahl der Gruppen (gre wegen alignment)
  CRDienst data;     // Erster Dienst
  char rest[1];       // ....
};

class Cacherecverw
{
  Cacherecord *ur;
  int buflen;
  CRDienst *aktd; // Aktueller Dienst
  int aktdnr;     // Dienstnummer fr Iteration		       

  u_char *ptr;    // Aktuelles Attribut in Attributiterator
  u_char *eptr;   // Zeiger hinter das letzte Attribut
public:
  // Initialisierungen
  Cacherecverw() { ur=NULL;eptr = NULL;ptr=NULL; }
  Cacherecverw(char *buffer,int len) { init(buffer,len); }
  void init(char *buffer,int len) { setbuffer(buffer,len); clear(); }
  void setbuffer(Cacherecord *cr);
  void setbuffer(char *buffer,int buflen);
  void clear() 
    {
      ur->ng=0;
      aktd = &(ur->data);
    }
  int totallen();
  Cacherecord *data() { return ur;  }
  int bufgross() { return buflen; } 
  //----- Iteratoren --------
  //   Dienstiteratoren
  CRDienst *ersterdienst(); 
  CRDienst *weitererdienst();
  CRDienst *findedienst(UINT4 dn);
   
  int lenreply(CRDienst *d) { return (d ? d->lang : 0); }
  
  auth_attr *erstes(CRDienst *d) { return (auth_attr *)((u_char *)ur+d->offset); }
  u_char *ende(CRDienst *d) { return (u_char *)ur+d->offset+d->lang; }
  // Attributiterator
  auth_attr *first(CRDienst *d);
  auth_attr *next();
  auth_attr *finde(u_char attribut);

  //--------- Aufbauiteratoren ------------
  void neuerdienst(UINT4 dn);
  void dienstcachetime(time_t t);
  int add(Attreintrag *attr,char *value);
  int add(u_char attribut,char *wert,int len);
  int add(u_char attribut,UINT4 w);
  int add(auth_attr *a);
  int addm(auth_attr *ma,int len);

  //--------- Operationen-----------------
  void copydienst(CRDienst *fdienst,Cacherecverw *cr2);
  void copyreplace(u_char attribut,char *wert,int len,UINT4 d,Cacherecverw *cr2);
  void mcopyreplace(auth_attr *a,int len,UINT4 d,Cacherecverw *cr2);

  //--------- Ausgabe --------------------  
  void print(ostream &ostr); // bis auf Dienst 0
  void print(const char *username,ostream &ostr); // invers zu parser
};

#endif


