/***********************************************************************
 * Modul.....: userscache.cc                                           *
 * Zweck.....: verwaltet einen cache-record                            *
 * Author....: Rudolf Weber informatikberatung Ravensburg              *
 * Copyright.: 1997 GNU_Copyleft                                       *
 ***********************************************************************/
#include<iostream.h>
#include<ctype.h>
#include<assert.h>
#include<stdio.h>
#include"dict.h"
#include"config.h"
#include"conf.h"
#include"usersparse.h"
#include"zeit.h"

Userservicescanner::Userservicescanner(Attrdict &dict):RaarScanner(dict)
{
  service = dict[PW_USER_SERVICE_TYPE];
  proto = dict[PW_FRAMED_PROTOCOL];
  zustand = 0;
  assert(service != NULL && proto != NULL);
}

int Userservicescanner::getpaar(Cacherecverw *urv)
{
    Attreintrag *at;
    char value[256];
    int r;
    if((r=scanpaar(at,value))>=0)
    {
      urv->add(at,value);
    }
    return r;
}


const char SERVICE[]="Service(";

int Userservicescanner::nextline()
{
  char *ret;
  if(lesezustand == 1) // Zeile wird nochmals gelesen
  {
    lesezustand=0;
    return zustand;
  }
  ret = getline();
  if(ret != NULL)
  {
    switch(puffer[0])
    {
    case '#': return KOMMZEILE;
    case ' ':
    case '\t':
      while(isspace(*p)) p++;
      zustand= (*p == '\0' && *p == '\n') ?  LEERZEILE : EINRUCKZEILE;
      break;
    default:
      zustand= BEGINNZEILE;
    }
    if(strncasecmp(p,SERVICE,sizeof(SERVICE)-1)==0)
    {
      p+=sizeof(SERVICE)-1;
      zustand=SERVICEZEILE;
    }
    else if(*p=='{') zustand=BLOCKBEGIN;
    else if(*p=='}') zustand=BLOCKENDE;
    return zustand;
  }
  else return ENDE;
}

UINT4 Userservicescanner::getdienst()
{
  UINT4 ret;
  const char *err = NULL;

  err = parsedienst(ret,p,service,proto);
  if(err != NULL)
  {
    error(err);
    while(*p!=')' &&  *p!='\n' && *p!='\0') p++;
  }
  if(*p!=')') error("Syntaxerror: ')' erwartet");
  else
  {
    p++;
    while(*p!='{' && *p!='\n' && *p!='\0') p++;
    if(*p=='{') zustand = BLOCKBEGIN;
  }
  return ret;
}

const char *PDIENST_UNGUETIGER_SERVICE="Ungltiger Service";
const char *PDIENST_KEIN_ATTRIBUT     ="Zusatzattribut fehl am Platz";
const char *PDIENST_UNGUELTIGES_ATTRIBUT = "Ungltiges Serviceattribut";

const char *parsedienst(UINT4 &ret,char *&p,Attreintrag *service,Attreintrag *proto)
{
  UINT4 uiproto;
  char buf[32];
  char *pz=buf;
  while(*p!=')' && *p!=',' && *p!='/' && *p!='\n' && *p!='\0') *pz++=*p++;
  *pz='\0';
  ret = service->wert(buf);
  if(ret == (UINT4)(-1)) return PDIENST_UNGUETIGER_SERVICE;
  if(*p==',' || *p=='/')
  {
    if(ret == PW_LOGIN_USER || ret == PW_DIALBACK_LOGIN_USER || ret == PW_SHELL_USER)
    {
      return PDIENST_KEIN_ATTRIBUT;
    }
    else
    {
      p++;
      pz=buf;
      while(*p!=')' && *p!='\n' && *p!='\0') *pz++=*p++;
      *pz='\0';
      uiproto=proto->wert(buf);
      if(uiproto != (UINT4)(-1))
      {
	ret |= uiproto << 16;
      }
      else return PDIENST_UNGUELTIGES_ATTRIBUT;
    }
  }
  return (const char *)0;
}

#if 0
UINT4 Userservicescanner::getdienst()
{
  char buf[32];
  char *pz=buf;
  UINT4 ret;
  while(*p!=')' && *p!=',' && *p!='\n' && *p!='\0') *pz++=*p++;
  *pz='\0';
  ret = service->wert(buf);
  if(ret == (UINT4)(-1)) { error("Ungltiger Service");return (UINT4)(-1);}
  if(*p++==',')
  {
    if(ret == PW_LOGIN_USER || ret == PW_DIALBACK_LOGIN_USER || ret == PW_SHELL_USER)
    {
      error("Zusatzattribut fehl am Platz");
      while(*p!=')' &&  *p!='\n' && *p!='\0') p++;
    }
    else
    {
       UINT4 r;
       pz=buf;
       while(*p!=')' && *p!='\n' && *p!='\0') *pz++=*p++;
       *pz='\0';
       r=proto->wert(buf);
       if(r != (UINT4)(-1))
       {
	 ret |= r << 16;
       }
       else error("Ungltiges Serviceattribut");
    }
    if(*p!=')') error("Syntaxerror: ')' erwartet");
    else
    {
      p++;
      while(*p!='{' && *p!='\n' && *p!='\0') p++;
      if(*p=='{') zustand = BLOCKBEGIN;
    }
  }
  return ret;
}
#endif

int Userservicescanner::readreplylist(Cacherecverw *urv)
{
   int ret;
   
   // Lese Reply-liste
    while(nextline() == EINRUCKZEILE)
    {
	do
	{
	    eatwhite();
	    if((ret = getpaar(urv))<0) return ERROR;
	    eatwhite();
	    if(*p != ',' && *p != '\n' && *p!='\0') 
	    {
		error("',' oder zeilenende erwartet");
		return ERROR;
	    }
	} while(*p++ == ',' && eatwhite() != '\0');
    }
    return OK;
}

const UINT4 DEFAULTDIENST=(PW_PPP << 16)|PW_FRAMED_USER;

int Userservicescanner::parsedienste(Cacherecverw *urv)
{
  int r;
  
  r=nextline();
  if(r==SERVICEZEILE)
  {
    UINT4 dienst;
    while(r==SERVICEZEILE)
    {
      if((dienst = getdienst())==UINT4(-1)) return ERROR;
      urv->neuerdienst(dienst);
      if(zustand != BLOCKBEGIN) r=nextline();
      if(zustand == SERVICEZEILE
	 || zustand==LEERZEILE) continue; // Service ohne Attribute
      if(zustand != BLOCKBEGIN) 
      {
	error("'{' erwartet ");
      }
      if(readreplylist(urv)!=OK) return ERROR;
      if(zustand != BLOCKENDE) 
      {
	error("'}' erwartet");
	return ERROR;
      }
      r=nextline();
    }
    return OK;
  }
  else if(r==EINRUCKZEILE)
  {
    urv->neuerdienst(DEFAULTDIENST);
    reset();
    return readreplylist(urv);
  }
  else
  {
    error("Dienst erwartet !");
    return ERROR;
  }
  
}

int Userprofscanner::parseuserprofile(char *name,Cacherecverw *urv)
{
    int ret;
    int zr;
    
    if((zr = nextprofile()) != BEGINNZEILE) return (zr==ENDE ? ENDE : ERROR);
    urv->neuerdienst(0);
    getstring(name); // username

    if(*p == '\n' || *p=='\0')
    {
	cerr << "Keine Checkliste bei " << name << endl;
    }
    
    // lese Checkliste
    do
    {
	eatwhite();
	if((ret = getpaar(urv))<0) return ERROR;
	eatwhite();
        if(*p != ',' && *p != '\n' && *p!='\0') 
	{
	    error("',' oder Zeilenende erwartet");
	    return ERROR;
	}
    } while(*p++ == ','); 

    return parsedienste(urv);
    
 
}

int Usertemplate::genprofile(char *pass,char *outdate,Cacherecverw *urv)
{
  urv->add(PW_PASSWORD,pass,strlen(pass));
  if(outdate != NULL)
  {
    Datum exp;
    const char *p;
    const char *ret;
    if((ret = exp.parse(outdate,&p))!= NULL)
    {
	cerr << "Datum falsch: " << ret << endl;
        return -1;
    }
    UINT4 exps = (UINT4)sekzeit696(exp);
    urv->add(ASCEND_PW_EXPIRATION,exps);
  }
  readreplylist(urv);
  return 0;
}






