/**********************************************************
 * Modul......: radiuslogin.cc                            *
 * Zweck......: Radiusloginbehandlung                     *
 * Copyright..: GNU                                       *
 *  Reimplementierung von Rudolf Weber Ravensburg         *
 **********************************************************/

#include<iostream.h>
#include<ctype.h>
#include<string.h>
#include"logmsg.h"
#include"config.h"
#include"radius.h"
#include"radiusclient.h"
#include"clientmessages.h"
#include"radlogin.h"

const int RCVBUF_SIZE=8192;
const int SNDBUF_SIZE=4096;
char recvbuffer[RCVBUF_SIZE];
char sendbuffer[SNDBUF_SIZE];

extern Attrdict dict;
extern Clientconf clientcf;
extern int debugflags;

LFUNC auth_radius(UINT4 client_port, char *username, char *passwd,Env &environ)
{
  UINT4  service, ftype, ctype;
  char   *p, username_realm[256];
  char   *realm;
  int    result;
  int    passlen;
  LFUNC  lfunc;
  u_char cryptpw[AUTH_PASS_LEN+1];
  struct sockaddr_in absenderadr;
  Zusenden ar(sendbuffer,SNDBUF_SIZE);
  Empfangen antwort(absenderadr,recvbuffer,RCVBUF_SIZE);
  Clientsecurityinfo csi;
  struct sockaddr_in salocal;
  int sockfd;

  /*--------- Bestimme den Service-Type --------------*/
#ifdef SCP
  /* determine based on the username what kind of service is requested.
     this allows you to use one password for all accounts, but the
     Merit radiusd supplies you just with the right information you
     need for the specified service type  -lf, 03/15/96 */
  
  switch (*username)
  {
  case 'S':
    service = PW_FRAMED_USER;
    ftype = PW_SLIP;
    ctype = 0;
    username++;
    break;
  case 'C':
    service = PW_FRAMED_USER;
    ftype = PW_SLIP;
    ctype = PW_VAN_JACOBSEN_TCP_IP;
    username++;
                                break;
  case 'P':
    service = PW_FRAMED_USER;
    ftype = PW_PPP;
    ctype = 0;
    username++;
    break;
  default:
    service = PW_LOGIN_USER;
    ftype = 0;
    ctype = 0;
    break;
  }
#else
  service = PW_LOGIN_USER;     
  ftype = 0;
  ctype = 0;
#endif

  /*---------------- Username mit default_realm----------*/
  strncpy(username_realm, username, sizeof(username_realm));
  
  /* Append default realm */
  char *default_realm = clientcf.default_realm;
  realm = strchr(username_realm, '@');
  if(realm == NULL)
  {
    if(default_realm && ((*default_realm) != '\0'))
    {
      strncat(username_realm, "@", sizeof(username_realm));
      strncat(username_realm, default_realm, sizeof(username_realm));
      realm = default_realm;
    }
    else
    {
      realm="";
    }
  }
  else realm++; // Vernichte @

  /*---------------- Baue Nachricht zusammen ------------*/
  sockfd = opensocket(salocal);
  salocal.sin_addr = myaddr();
  csi.secret = clientcf.secinf.secret(salocal.sin_addr,realm);
  if(csi.secret == NULL)
  {
    cerr << "Von Rechner " << salocal << " ist eine Abfrage nicht erlaubt!\n";
    exit(1);
  }
  if(debugflags)
  {
    cout <<  "Absender " << salocal
#ifdef DEBUG
	 <<  " Secret: " << csi.clinfo->secret
#endif
	 << endl;
  }

  random_vector(csi.vector);
  ar.initsend(PW_AUTHENTICATION_REQUEST,(u_char)seqnr(clientcf.seqfile),csi.vector);
  ar.addattr(PW_CLIENT_ID,ntohl((UINT4)(salocal.sin_addr.s_addr)));
  ar.addattr(PW_CLIENT_PORT_ID,client_port);
  ar.addattr(PW_USER_SERVICE_TYPE,service);
  ar.addattr(PW_USER_NAME,username_realm,strlen(username_realm));
  if(ftype != 0)  ar.addattr(PW_FRAMED_PROTOCOL,ftype);
  if(ctype != 0)  ar.addattr(PW_FRAMED_COMPRESSION,ctype);
  ar.calc_digest(csi.secret);
  passlen = strlen(passwd);
  ciffrepassword(cryptpw,(u_char *)passwd,passlen,ar.getdigest());
  ar.addattr(PW_PASSWORD,(char *)cryptpw,passlen);

  /*---------------- senden --------------------*/
  if(!komuwithserver(sockfd,ar,antwort,clientcf.radhosts,csi,clientcf))
  {
    return (LFUNC)NULL;
  }
  if(debugflags > 0)
  {
      cout << "Erhaltene Rckmeldung:\n";
      antwort.print(cout);
      cout << "\n";
  }
  if(antwort.code()==PW_AUTHENTICATION_ACK)
  {
    char name[256];
    char value[1024];
    char *p;
    auth_attr *akt;
    environ.add("RADIUS_USER_NAME", username);
    /*------- Trage die Attribute in die Umgebung ein----*/
    for(akt = antwort.first();akt !=NULL;akt = antwort.next())
    {
      Attreintrag *attr=dict[akt->id];
      if(attr==NULL)
      { logmsg(LOG_WARNING,"Unbekanntes Attribut %u in Antwort\n",akt->id);
        continue;
      }
      strcpy(name,"RADIUS_");
      attr->string(name+7,value,akt->data,akt->len);
      /* Translate "-" => "_" and uppercase*/
      for(p = name+7; *p; p++)
      {
	*p = toupper(*p);
	if (*p == '-') *p = '_';
      }
      environ.add(name,value);

      switch(akt->id)
      {
      case PW_USER_SERVICE_TYPE: service = getintval(akt);
	break;
      case PW_FRAMED_PROTOCOL: ftype = getintval(akt);
	break;
      case PW_FRAMED_COMPRESSION: ctype = getintval(akt);
	break;
      default:
	;
      }
    }
    if(service == PW_FRAMED_USER)
    {
      if(ftype == PW_PPP)
      {
	lfunc = start_ppp;
      }
      else if(ftype == PW_SLIP)
      {
	lfunc = start_slip;
      }
    }
    else
    {
      lfunc = radius_login;
    }
    return lfunc;
  }
  else // Ablehnen
  {
    char msg[1024];
    auth_attr *pmattr = antwort.finde(PW_PORT_MESSAGE);
    msg[0]='\0';
    if(pmattr != NULL)
    {
      getstring(msg,pmattr);
    }
    logmsg(LOG_NOTICE, "authentication FAILED, type RADIUS, username %s: %s\n", username_realm,msg);
    if (msg[0] != '\0')
      printf(SC_SERVER_REPLY, msg);
    else
      printf(SC_RADIUS_FAILED);

    return (LFUNC)NULL; 
  }
}

void radius_login(char *username,Env &environ)
{
  char *login_radius = clientcf.login_radius;
  
  execle(login_radius, login_radius, NULL, environ.export());
  
  logmsg(LOG_ERR, "couldn't execute %s: %s", login_radius, strerror(errno));
  fprintf(stderr, "couldn't execute %s: %s", login_radius, strerror(errno));
  
  sleep(1);       /* give the user time to read */
  exit(-1);
}

void start_slip(char *username,Env &environ)
{
  fprintf(stderr,"Wer benutzt noch SLIP ? Ich nicht \n");
  fprintf(stderr,"Bitte in radlogin implmentieren lassen \n");
  sleep(2);
  exit(-1);
}

void start_ppp(char *username,Env &environ)
{
  char *pppauf = clientcf.start_ppp;
  char *pppcommand;
  char *arg[20];
  int i=0;
  arg[0]=pppcommand=strtok(pppauf," \n");
  
  while(i<20 && arg[i]!=NULL)
  {
    i++;
    arg[i]=strtok(NULL," \n");
  }
  syslog(LOG_NOTICE,"execve %s ",pppauf);
  execve(pppcommand,arg,environ.export());
  logmsg(LOG_ERR, "couldn't execute %s: %s", pppauf, strerror(errno));
  fprintf(stderr, "couldn't execute %s: %s", pppauf, strerror(errno));
  
  sleep(1);       /* give the user time to read */
  exit(-1);
}
