/**************************************************************************
 * Programm...: radacct                                                   *
 * Zweck......: RADIUS_Accounting-Frondend                                *
 * redisigned/rehackt from Lars Fenebergs radiusclients to build in the   *
 * famous Radiusclasses                                                   *
 * copyright: 1997-1999: Rudolf Weber Ravensburg                          *
 * email: rw@rv.infnet.verein.de                                          *
 * Rights: see GNU-Copyleft (Distribution only with Source-Code)          *
 * Comericial Users are ask to send money                                 *
 **************************************************************************/
#include<iostream.h>
#include<getopt.h>
#include"logmsg.h"
#include"radius.h"
#include"config.h"
#include"ttyidmap.h"
#include"radiusclient.h"
#include"racctascanner.h"
static char	rcsid[] = "Rudolfs Faelschung ";

Clientconf clientcf(defaultclientconfigfile());
Attrdict dict;
Ttyidmap ttyidmap;

char *progname;
int debugflags=0;
char buffer[1024];
char recvbuffer[1024];
struct sockaddr_in absenderadr;
Zusenden msg(buffer,1024);
Empfangen antwort(absenderadr,recvbuffer,1024);

void usage(void)
{
  cerr << "Usage: " << progname << "[-Vhd] [-i <client_port>] [-t msgtyp]\n";
  cerr << "  -i            ttyname to send to the server\n";
  cerr << "  -V            output version information\n";
  cerr << "  -h            output this text\n";
  cerr << "  -d            debugging \n";
  cerr << "  -t            Messagetype \n";
  exit(-1);
}

void version(void)
{
  cerr << progname << " " << rcsid << endl;
  exit(-1);
}

int main(int argc,char **argv,char **env)
{
   extern char *optarg;
   extern int optind;
   char *dictdir = clientcf.dictpath;
   char *clientspath = clientcf.clientpath;
   char *ttyidmappath = clientcf.mapfile;
   UINT4 client_port = 0;
   RacctaScanner racctascan(dict);
   Attreintrag *at;
   char value[128];
   Clientsecurityinfo csi;
   Server *hosts;
   struct sockaddr_in salocal;
   int sockfd;
   int c;
   struct option long_options[] =
   {   {"dict",1,0,1 },
       {"conf",1,0,'c'},
       {"nonrandom",1,0,2},
       {"null",1,0,'0'},
       {0,0,0,0}
   };
   int oi = 0;
   char user[256];
   char realm[256];
   auth_attr *ur,*pwdr;
   u_char msgtyp = PW_ACCOUNTING_REQUEST;
   int nr=0;
   progname=argv[0];
   logmsginit(LOGMSG_STDERR,progname,0,0);
   

   if(ttyidmap.lese(ttyidmappath)<0) exit(-1);
  
   while((c = getopt_long(argc, argv, "c:i:hVdt:H:",long_options,&oi)) > 0)
   {
     switch(c)
     {
     case 'i':
       client_port = ttyidmap[optarg];
       break;
     case 'V':
       version();
       break;
     case 'h':
       usage();
       break;
     case 'd':
       debugflags=1;
       break;
     case 't':
             msgtyp = (u_char)atoi(optarg);
	     if(msgtyp==0)
	     {
	       msgtyp=str2msgtype(optarg);
             }
             if(msgtyp==0) // immer noch
	     {
	       cerr << "Zulssige Messagetypen:" << endl;
	       listmsgtypes(cerr);
	       exit(1);
             }
             break;
     case 1:
	   dictdir = optarg;
	   cout << "Dict: " << dictdir << endl;
	   break;
     case 2:
           {
	       if(strlen(optarg)==AUTH_VECTOR_LEN)
	       {
		   nr=1;
		   u_char *pz=csi.vector;
		   for(u_char *po=(u_char *)optarg;
		       po<(u_char *)optarg+AUTH_VECTOR_LEN;*pz++=*po++);   
	       }
	       else
	       {
		   cerr << "Authorisierungsvektor mu 16 Zeichen lang sein\n";
		   exit(1);
	       }
           }
     case 'c':
         clientcf.readconfig(optarg);
	 dictdir = clientcf.dictpath;
	 clientspath = clientcf.clientpath;
	 ttyidmappath = clientcf.mapfile;
	 break;	     
     case 'H':
       clientcf.addradserver(optarg);
       clientcf.addacctserver(optarg);
       break;
     default:
       exit(-1);
       break;
     }
   }
   dict.init(dictdir);
   if(client_port == 0)
   {
     /* we take stdout here, because stdin is usually connected
      * to our input file
      */
     client_port = ttyidmap[ttyname(1)];
   }

 
   sockfd = opensocket(salocal);
   salocal.sin_addr = myaddr();
   
   memset((char *)(csi.vector),'\0',AUTH_VECTOR_LEN);
   msg.initsend(msgtyp,(u_char)seqnr(clientcf.seqfile),csi.vector);
   msg.addattr(PW_CLIENT_ID,ntohl((UINT4)(salocal.sin_addr.s_addr)));
   msg.addattr(PW_CLIENT_PORT_ID,client_port);
   while(racctascan.getattribut(at,value)>0)
   {
     msg.addattr(at,value);
   }
   ur=msg.finde(PW_USER_NAME);
   if(ur!=NULL)
   {
     getuserandrealm(user,realm,ur);
   }
   else
   {
     realm[0]='\0';
   }
   csi.secret  = clientcf.secinf.secret(salocal.sin_addr,realm);
   if(csi.secret == NULL)
   {
     cerr << "Von Rechner " << salocal << " mit realm " << realm << " ist eine Abfrage nicht erlaubt!\n";
     exit(1);
   }

   

   // Accounting anders
   if(msgtyp == PW_ACCOUNTING_REQUEST)
   {
     msg.calc_responsedigest(csi.secret);
     memcpy(csi.vector,msg.getvector(),AUTH_VECTOR_LEN);
     hosts = clientcf.radaccthosts;
   }
   else
   {
     random_vector(csi.vector);
     msg.setvector(csi.vector);
     msg.calc_digest(csi.secret);
     hosts = clientcf.radhosts;
   }
   if((pwdr = msg.finde(PW_PASSWORD))!=NULL)
   {
     u_char cryptpw[AUTH_PASS_LEN];
     ciffrepassword(cryptpw,pwdr->data,(int)(pwdr->len)-2,msg.getdigest());
     memcpy(pwdr->data,cryptpw,(int)(pwdr->len)-2);
   }
   if(debugflags)
   {
     cout << "zu sendende Anforderung:\n";
     msg.print(cout);
   }
   if(!komuwithserver(sockfd,msg,antwort,clientcf.radaccthosts,csi,clientcf))
   {
     cerr << "Anfrage ist Fehlgeschlagen \n";
     exit(1);
   }
   if(msgtyp != PW_ACCOUNTING_REQUEST || debugflags)
   {
     cout << "Erhaltene Rckmeldung:\n";
     antwort.print(cout);
   }
}








