/**************************************************************************
 * Programm...: radlogin                                                  *
 * Zweck......: RADIUS_Replacement for login                              *
 * redisigned/rehackt from Lars Fenebergs radiusclients to build in the   *
 * famous Radiusclasses                                                   *
 * copyright: 1997: Rudolf Weber Ravensburg                               *
 * email: rw@w-4.de                                                       *
 * Rights: see GNU-Copyleft (Distribution only with Source-Code)          *
 * Comericial Users are ask to send money                                 *
 **************************************************************************/
#include<iostream.h>
#include<sys/file.h>
#include<sys/utsname.h>
#include<getopt.h>
#include<signal.h>
#include"logmsg.h"
#include"radius.h"
#include"config.h"
#include"ttyidmap.h"
#include"radiusclient.h"
#include"radlogin.h"
#include"env.h"
#include"rcio.h"
#include"clientmessages.h"

static char	rcsid[] = "Rudolfs Faelschung ";
void detect_telnet(void); /*aus outbound.cc*/

Clientconf clientcf(defaultclientconfigfile());;
Attrdict dict;
Ttyidmap ttyidmap;

char *progname;
int debugflags=0;
extern int keepalive; // outbound.c
static int login_timeout; // Kommunikation mit Timeout-Handler

static char *subst_placeholders (char *str, char *tty)
{
  char *p, *q;
  static char buf[4096];
  struct utsname uts;

  uname (&uts);

  p = str;
  q = buf;

  while (*p != '\0')
  {
    switch (*p)
    {
    case '\\':
      if (*(p + 1) == '\0')
	break;
      p++;
      switch (*p)
      {
      case 'I':
	strcpy (q, rcsid);
	q += strlen (rcsid);
	break;
      case 'L':
      case 'P':
	strcpy (q, tty);
	q += strlen (tty);
	break;
      case 'S':
	strcpy (q, uts.sysname);
	q += strlen (uts.sysname);
	break;
      case 'N':
	strcpy (q, uts.nodename);
	q += strlen (uts.nodename);
	break;
      case 'R':
	strcpy (q, uts.release);
	q += strlen (uts.release);
	break;
      case 'V':
	strcpy (q, uts.version);
	q += strlen (uts.version);
	break;
      case 'M':
	strcpy (q, uts.machine);
	q += strlen (uts.machine);
	break;
      case 'D':
	strcpy (q, uts.domainname);
	q += strlen (uts.domainname);
	break;
      case '\\':
	*q = '\\';
	q++;
	break;
      }
      break;
    case '@':
      strcpy (q, uts.nodename);
      q += strlen (uts.nodename);
      break;
    case '\n':
      strcpy (q, "\r\n");
      q += 2;
      break;
    default:
      *q = *p;
      q++;
      break;
    }

    p++;
  }
  *q = '\0';

  return buf;
}

static int login_allowed(char *tty)
{
  FILE *fp;
  char fname[PATH_MAX];
  int c;

  strcpy(fname, clientcf.nologin);
  if(access(fname, F_OK) < 0)
  {
    if(tty)
    {
      sprintf(fname, "%s.%s", clientcf.nologin, tty);
      if(access(fname, F_OK) < 0)
	return 1;
    }
    else
    {
      return 1;
    }
  }

  if((fp = fopen(fname, "r")) != NULL)
  {
    while((c = fgetc(fp)) != EOF)
    {
      if(c == '\n')
	fputc('\r', stdout);
      fputc(c, stdout);
    }
    fflush(stdout);
    fclose(fp);
  }
  else
  {
    printf(SC_NOLOGIN);
  }
  return(0);
}

static void usage(void)
{
  cerr << "Usage:" << progname << "[-Vh] [-i <client_port>]\n";
  cerr << "  -i		ttyname to send to the server\n";
  cerr << "  -n		turn off keepalive on sockets\n";
  cerr << "  -d		turn off automatic telnet detection\n";
  cerr << "  -V		output version information\n";
  cerr << "  -h		output this text\n";     
  exit(-1);
}

static void version(void)
{
  cerr << progname << " Rudolfs radius 0.4\n";
  exit(-1);
}

static void alarm_handler(int sn)
{
  extern int login_timeout; 
  fprintf(stderr, SC_TIMEOUT,login_timeout);
  sleep(1);
  exit(-1);	
}

int main(int argc,char **argv,char **env)
{
   extern char *optarg;
   extern int optind;
   char *dictdir = clientcf.dictpath;
   char *clientspath = clientcf.clientpath;
   char *ttyidmappath = clientcf.mapfile;
   extern char *port_pool;   // outbound.cc hack 
   port_pool = clientcf.port_pool;
   login_timeout = clientcf.logintimeout;
   Env environ(ENV_SIZE);
   void (*login_func)(char *,Env &);
   /* Operationsparameter */
   int notelnet = 0;
   int noissue = 0;
   int tries, remaining, maxtries, c;

   char	 username[128];
   char	 passwd[AUTH_PASS_LEN + 1];
   UINT4 client_port = 0;
   int is_tty;
   char	tty[1024];            // tty-Name
   char	*ttyn;

   progname=argv[0];
   logmsginit(LOGMSG_STDERR,progname,0,0);
   dict.init(dictdir);
   if(ttyidmap.lese(ttyidmappath)<0) exit(-1);
   
   is_tty = isatty(0);
   *tty='\0';
   client_port=0;
   ttyn = (is_tty ? ttyname(0) : (char *)0 );

   while ((c = getopt(argc,argv,"i:dtnhVD")) > 0)
   {
     switch(c)
     {
     case 'i':
       ttyn = optarg;
       break;
     case 'd':
       keepalive = 0;
       break;
     case 't':
       notelnet = 1;
       break;
     case 'n':
       noissue = 1;
       break;
     case 'V':
       version();
       break;
     case 'h':
       usage();
       break;
     case 'D': debugflags=1; break;
     default:
       exit(-1);
       break;
     }
   }
   
   
   if(!is_tty && ttyn == NULL) // kein terminal, muss socket sein !
   {
     struct sockaddr_in sin;
     int sinlen = sizeof(sin);
     if (getpeername(0, (struct sockaddr *)&sin, &sinlen) < 0
	 && sin.sin_family == AF_INET)
     {
       sprintf(tty,"%s:%d", ip2hostname(ntohl(sin.sin_addr.s_addr)),
	       ntohs(sin.sin_port));
       client_port = 0;
     } 
   }
   else
   { 
     char *p;
     if ((p = strrchr(ttyn, '/')) == NULL)
       strncpy(tty, ttyn, sizeof(tty));
     else
       strncpy(tty, p+1, sizeof(tty));
     
     client_port = ttyidmap[ttyn];
   }
   if(debugflags>0)
   {
     cout << "tty-Name: " << tty << " client_port: " << client_port << endl;
   }
  
   if ((argc - optind) == 1)
   {
     strncpy(username,argv[optind], sizeof(username));
   }
   else
   {
     if (!is_tty && !notelnet)
       detect_telnet();

     *username = '\0';
     if (!noissue)
     {
       FILE *fp;
       char buf[4096];
       if (clientcf.issue !=NULL
	   && ((fp = fopen(clientcf.issue, "r")) != NULL))
       {
	 while (fgets(buf, sizeof(buf), fp) != NULL)
	   fputs(subst_placeholders(buf, tty), stdout);
	 
	 fflush(stdout);
	 fclose(fp);
       }
       else
       {
	 fputs(subst_placeholders(DEFAULT_ISSUE, tty), stdout);
	 fflush(stdout);
       }
     }
   }
#ifdef SECURITY_DISABLED
   environ.import(env);
#else
   environ.add("IFS"," ");
   environ.add("PATH",clientcf.default_path);
#endif

   /*-------- get login and passwort -----------*/
   remaining = login_timeout;
   maxtries =  clientcf.maxlogintries;
   tries = 1;
   while (tries <= maxtries)
   {
     char *ret;
     alarm(remaining);
     while (!*username)
     {
       ret = rc_getstr(username,sizeof(username)-1,SC_LOGIN, 1);
       if (!ret) exit (-1);
     }
     ret = rc_getstr(passwd,sizeof(passwd)-1,SC_PASSWORD, 0);
     if (!ret) exit (-1);
     
     remaining = alarm(0);

     if (clientcf.authorder & AUTH_LOCAL_FST)
     {
       if (is_tty)
	 login_func = auth_local(username, passwd,environ);
       
       if (!login_func)
	 if (clientcf.authorder & AUTH_RADIUS_SND)
	   login_func = auth_radius(client_port, username, passwd,environ);
     }
     else
     {
       login_func = auth_radius(client_port, username, passwd,environ);
       if (!login_func)
	 if ((is_tty) && clientcf.authorder & AUTH_LOCAL_SND)
	   login_func = auth_local(username, passwd,environ);
     }

     memset(passwd, '\0', sizeof(passwd));

     if (login_func != NULL)
     {
       if (login_allowed(tty))
       {
	 (*login_func)(username,environ);
       }
       else
       {
	 sleep(1);
	 exit (-1);
       }
     }
     *username = '\0';
     

     if ((++tries) <= maxtries)
     {
       alarm(remaining);
       sleep(tries * 2);
       remaining = alarm(0);
     }
   }
   fprintf(stderr, SC_EXCEEDED);
   sleep(1);
   
   exit (-1);
}













