/*****************************************************************
 * Modul.....: admin_realm.cc                                    *
 * Zweck.....: Implementiert Klasse admin_realm                  *
 * Author....: Rudolf Weber Informatikberatung Ravensburg        *
 * Copyright.: 1998 GNU                                          *
 *****************************************************************/
#include<iostream.h>
#include<string.h>
#include<stdlib.h>
#include<unistd.h>
#include<errno.h>
#include<sys/time.h>
#include<ctype.h>
#include<syslog.h>
#include"radius.h"
#include"radlog.h"
#include"admin_realm.h"
#include"config.h"
extern Realms        realms;
extern PaketLogfile  paketlog;
extern PaketLoglevel pll;
extern int           plaktiv;
extern int debugflags;

int Admin_realm::radauthenticate(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  Admin_realm *dr=(Admin_realm *)r;
  return dr->radauthenticate(sockfd,msg,antwort);
}

int Admin_realm::accounting(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  return realms.getdefaultrealm()->accounting(sockfd,msg,antwort);
}

int Admin_realm::passchange(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  return realms.getdefaultrealm()->passchange(sockfd,msg,antwort);
}

#ifdef RADADMINCOMPAT
int Admin_realm::adminrequest(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  // Schiebe Auftrag an defaultrealm weiter, falls er das kann.
  Realm *dflr = realms.getdefaultrealm();
  Raktion fktp;

  if((fktp=dflr->aktion(msg.code()))!=NULL)
  {
    return (*fktp)(dflr,sockfd,msg,antwort);
  }
  else
  {
    char kleinerplatz[32];
    const char *codestr;
    if((codestr=msgtype2str(msg.code()))==NULL)
    {
      snprintf(kleinerplatz,32,"%d",(int)(msg.code()));
      codestr = kleinerplatz;
    }
    syslog(LOG_ERR,"Realm %s bearbeitet %s nicht",dflr->name(),codestr);
    
    return -1;
  }
}
#endif

int Admin_realm::init(char *parameterstr)
{
  initaktionen(); //Aufbau der Tabelle 
  routetab.init(ROUTETABPATH);
  return 0;
}

int Admin_realm::anfang()
{
  return 0;
}

int Admin_realm::ende()
{
  return 0;
}

void Admin_realm::initaktionen()
{
  aktionen[PW_AUTHENTICATION_REQUEST]=Admin_realm::radauthenticate;
  aktionen[PW_ACCOUNTING_REQUEST]=Admin_realm::accounting;
  aktionen[PW_PASSWORD_REQUEST]=Admin_realm::passchange;
#ifdef RADADMINCOMPAT
  aktionen[PW_RW_NEWUSER_REQUEST]=Admin_realm::adminrequest;
  aktionen[PW_RW_NEWUSER_REJECT]=Admin_realm::adminrequest;
  aktionen[PW_RW_USERUPD_REQUEST]=Admin_realm::adminrequest;
#endif
}

int Admin_realm::radauthenticate(int sockfd,Empfangen &msg,Zusenden &antwort)
{
  extern int debugflags;
  auth_attr *ur;
  const Clientinfo *clinfo;
  char clearpwd[128];
  extern Clients  clients;

  clinfo = clients.finde(msg.getabsender().sin_addr,name());
  if(clinfo == NULL) 
  {
      
      if(debugflags) cerr << "Unbekannter Client " << endl;
      syslog(LOG_INFO,"Unbekannte Clientaddresse %s fr realm %s",
	     iptostr(clearpwd,msg.getabsender().sin_addr),name());
      return RJ_UNKNOWN_CLIENT;
  }
  msg.setsecret(clinfo->secret);
  msg.calc_digest(clinfo->secret);
  ur=msg.finde(PW_USER_NAME);
  if(ur == NULL)
  {
    char clid[65];
    if(plaktiv && pll == PLL_ADMIN)
    {
      paketlog.log(msg);
    }
    auth_attr *cid = msg.finde(PW_CALLER_ID);
    if(cid != NULL)
    {
      getstring(clid,cid);
      return authclid(clid,sockfd,msg,antwort);
    }
    else
    {
      // Nun so was, eine Anforderung ohne user .....
      char kleinerplatz[20];
      syslog(LOG_ERR,"No User-Name and no clid from %s\n",
	     iptostr(kleinerplatz,msg.getabsender().sin_addr));
      return -1;
    }
  }
  else
  {
    char user[256];
    char realm[256];
    int ret;
    // Prfung wurde schon gemacht, brauche nur user nochmal
    getuserandrealm(user,realm,ur);
    if((ret=administrativequery(user,sockfd,msg,antwort,clinfo))==0)
    {
      return realms.getdefaultrealm()->radauthenticate(sockfd,msg,antwort);
    }
    else
    {
      return ret;
    }
  }
}

int Admin_realm::administrativequery(char *user,int sockfd,Empfangen &msg,Zusenden &antwort,const Clientinfo *clinfo)
{
  char puffer[1024];
  int r;
  if(strncasecmp(user,"route-",6)==0)
  {
    if(plaktiv && pll == PLL_ADMIN)
    {
      paketlog.log(msg);
    }
    r=routetab.read(user,strlen(user),puffer,1024);
    if(r==1) 
    {
      char *tok;
      antwort.initsend(PW_AUTHENTICATION_ACK,msg.id(),msg.getvector());
      for(tok=strtok(puffer,";");tok!=NULL;tok=strtok(NULL,";"))
      {
	antwort.addattr(PW_FRAMED_ROUTE,tok,strlen(tok));
      }
    }
    else
    {
      antwort.initsend(PW_AUTHENTICATION_REJECT,msg.id(),msg.getvector());
    }
    antwort.calc_responsedigest(msg.getsecret());
    if(debugflags) antwort.print(cout);
    antwort.send(sockfd,msg.getabsender());
    return 1;
  }
  return 0;
}

int Admin_realm::authclid(char *clid,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  // Auffinden Callerid -> user@realm
  // Authorisierung (neues Konzept radiusdb ntig)

  syslog(LOG_NOTICE,"Clid is not implemented yet ");
  return -1;
}



