#include<iostream.h>
#include<string.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/time.h>
#include<pwd.h>
#include"errmsg.h"
#include"md5.h"
#include"auth.h"
#include"radlog.h"
#if 0
extern "C" char *crypt(char *key,char *salt);
#endif
extern int debugflags;
extern PaketLogfile  paketlog;
extern PaketLoglevel pll;
extern int           plaktiv;

const char PASSWD_FALSCH[]="Pawort falsch";
const char PASSWD_UNBKT[]="Unbekannte Codierung";

int auth_pass(const char *&portmsg,auth_attr *authpwd,char *clearpwd,char dbpwt,char *dbpwd,int dbpwlen,Empfangen& msg)
{
  const char *ret;
  if(authpwd->id == PW_PASSWORD)
  {
    ret = passcheck(clearpwd,dbpwt,dbpwd,dbpwlen);
    if(ret == NULL) return 1;
    else
    {
      if(plaktiv && pll == PLL_WRONGPW)
      {
	 paketlog.logstr("#Falsches Pawort:>%s<\n",clearpwd);
	 paketlog.log(msg);
      }	
      portmsg=ret;
      return 0;
    }
  }
  else /* PW_CHAP_PASSWORD */
  {
    if(dbpwt=='C')
    {
      char string[AUTH_PASS_LEN + 64];
      u_char pw_digest[CHAP_VALUE_LENGTH+1];
      char *ptr=string;
      int size = dbpwlen;
      int challen = AUTH_VECTOR_LEN;
      auth_attr *challattr = msg.finde(PW_CHAP_CHALLENGE);
      
      *ptr++ = authpwd->data[0];
      if(dbpwd[size-1]=='\0') size--;
      memcpy(ptr,dbpwd,size);
      ptr+=size;
      if(challattr!= NULL) 
      {
	challen=challattr->len-2;
	memcpy(ptr,(char *)(challattr->data),challen);
      }
      else
      {
	challen = AUTH_VECTOR_LEN;
	memcpy(ptr,msg.getvector(), challen);
      }
      md5_calc(pw_digest,(u_char *)string,1 + size + challen);
      if(memcmp(pw_digest,(char *)(authpwd->data)+1,CHAP_VALUE_LENGTH)==0)
      {
	return 1;
      }
      else
      {
	if(plaktiv && pll == PLL_WRONGPW)
	{	     
	    paketlog.log(msg);
	}	
	return 0;
      }
    }
    else
    {
      portmsg="Keine CHAP_Authorisierung mit verschlsseltem-Passwort!";
      return 0;
    }
  }
}


const char *passcheck(char *versuch,char pwt,char *passwd,int pwlen)
{
   switch(pwt)
   {
   case 'C': // uncodiertes passwort
     return (memcmp(versuch,passwd,pwlen)==0 ?
	     (const char *)NULL : PASSWD_FALSCH );
   case 'Y': // cpytet
     {
      char *encpw;
      /* Run encryption algorythm */
#if defined(SOLARIS)
      encpw = crypt((CONST char *)versuch, (CONST char *)passwd);
#else
      encpw = crypt(versuch,passwd);
#endif
      return (strcmp(encpw,passwd)==0 ? (const char *)NULL : PASSWD_FALSCH);
    }
   case 'U': // UNIX
     return unix_pass(passwd,versuch);// passwd ist Unix-User
   default:
     return PASSWD_UNBKT;
   }
}

#if 0
char *cryptpw(char *pw)
{
  char salt[3];
  time_t tm;
  
  time(&tm);
  salt[0] = (tm & 0x0f) +       'A';
  salt[1] = ((tm & 0xf0) >> 4) + 'a';
  salt[2] = '\0';

  return crypt(pw,salt);
}
#endif

/*************************************************************************
 *
 *	Function: unix_pass
 *
 *	Purpose: Check the users password against the standard UNIX
 *		 password table.
 *
 *************************************************************************/

const char *unix_pass(char *name,char *password)
{
  struct passwd	*pwd;
  char		*encpw;
  char		*encrypted_pass;
#if !defined(NOSHADOW)
#if defined(M_UNIX)
  struct passwd	*spwd;
#else
  struct spwd	*spwd;
#endif
#endif /* !NOSHADOW */
  
  /* Get encrypted password from password file */
  if((pwd = getpwnam(name)) == NULL) return UNIX_GETPWNAME_ERR;
  encrypted_pass = pwd->pw_passwd;
  
#if !defined(NOSHADOW)
  if(strcmp(pwd->pw_passwd, "x") == 0)
  {
    if((spwd = getspnam(name)) == NULL)  return UNIX_GETSHDWNAME_ERR;
#if defined(M_UNIX)
    encrypted_pass = spwd->pw_passwd;
#else
    encrypted_pass = spwd->sp_pwdp;
#endif	/* M_UNIX */
  }
#endif	/* !NOSHADOW */

  /* Run encryption algorythm */
#if defined(SOLARIS)
  encpw = crypt((CONST char *)password, (CONST char *)encrypted_pass);
#else
  encpw = crypt((char *)password,(char *)encrypted_pass);
#endif
  /* Check it */
  if(strcmp(encpw, encrypted_pass)) {
    return UNIX_BAD_PASSWORD;
  }
  return (const char *)0;
}

int auth_check_timeout(const char *&portmsg,long expt)
{
  int ok;
  struct timeval tv;
  gettimeofday(&tv,NULL);
  if(expt > tv.tv_sec+604800)  // eine Woche
  {
    ok = 1;
  }
  else if(expt > tv.tv_sec)
  {
    ok = 1;
    if(!portmsg) portmsg = "Pawortzeit < 7 Tage";
  }
  else
  {
    ok = 0;
    if(!portmsg) portmsg = "Pawort abgelaufen\n";
    if(debugflags) { cerr << "Pawort abgelaufen\n";}
  }
  return ok;
}








