/**************************************************************
 * Modul.....: caching_realm.cc                               *
 * Zweck.....: Implementiert Klasse Caching_realm             *
 * Author....: Rudolf Weber Informatikberatung Ravensburg     *
 * Copyright.: 1998 GNU                                       *
 **************************************************************/
#include<iostream.h>
#include<string.h>
#include<stdlib.h>
#include<unistd.h>
#include<errno.h>
#include<sys/time.h>
#include<ctype.h>
#include"logmsg.h"
#include"config.h"
#include"radius.h"
#include"md5.h"
#include"auth.h"
#include"caching_realm.h"
#include"clients.h"

const char NIEMAND_NEUES[]="niemand_neues";
const char CTO[] = "timeout = %ld";
extern int debugflags;
extern Attrdict dict;
extern Clients  clients;

int Caching_realm::radauthenticate(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  Caching_realm *dr=(Caching_realm *)r;
  return dr->radauthenticate(sockfd,msg,antwort);
}

int Caching_realm::accounting(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  return r->accounting(sockfd,msg,antwort);
}

int Caching_realm::passchange(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  return -1;
}


int Caching_realm::init(char *parameterstr)
{
  char *p;
  char *relname=parameterstr;
  char *keyword;
  struct sockaddr_in salocal;
  long ctoh=0;

  initaktionen(); // hier ist sind die virtuelle tabelle fertig aufgebaut
  niemandneues=0;
  cachetimeout=CACHETIMEOUT;
  /* Interpretiere Parameter */
  for(p=parameterstr;*p!='\0' && isalnum(*p);p++);
  while(isspace(*p)) p++;
  if(*p!='\0' && *p!=',') 
  { 
    throw Syntaxerror("Ungltiger Relationsname");
    return -1;
  }
  strcmp(dbname,relname);

  while(*p==',')
  {
    *p++='\0';
    keyword=p;
    while(isspace(*p)) p++;
    if(strncmp(p,NIEMAND_NEUES,sizeof(NIEMAND_NEUES)-1)==0)
    {
      keyword=NULL;niemandneues=1;p+=sizeof(NIEMAND_NEUES)-1;
    }
    else if(sscanf(keyword,CTO,&ctoh)==1)
    {
      keyword=NULL;
      cachetimeout=ctoh*3600;
    }
    while(*p!=',' && *p!='\0') p++;
    if(keyword!=NULL)
    {
      char oldp=*p;
      *p='\0';
      throw Syntaxerror("Ungltiges Schlsselwort %s",keyword);
      *p=oldp;
    }
  }
  clsock=opensocket(salocal);
  salocal.sin_addr = myaddr();
   
  csi.secret = clc.secinf.secret(salocal.sin_addr,name());
  if(csi.secret == NULL)
  {
    throw Syntaxerror("Keine Erlaubnis fr realm %s ",name());
    return -1;
  }
  return 0;
}
  
int Caching_realm::anfang()
{  
  try
  {
    db.init(dbname,UDB_READ);
  }
  catch(DbException &dbe)
  {
    throw Databaseerror(dbe.what());
    return -1;
  }

  return 0;
}

Caching_realm::ende()
{
  db.close();
  if(debugflags)
  {
    cerr << "Beende db_adminrealm " << name() << endl;
  }
}

void Caching_realm::initaktionen()
{
  aktionen[PW_AUTHENTICATION_REQUEST]=Caching_realm::radauthenticate;
  aktionen[PW_ACCOUNTING_REQUEST]=Caching_realm::accounting;
}

int Caching_realm::radauthenticate(int sockfd,Empfangen &msg,Zusenden &antwort)
{
  extern int debugflags;
  char user[256];
  char realm[256];
  char clearpwd[256];
  auth_attr *authpwd=NULL; // Zeiger auf das Passwort
  auth_attr *ur;
  char recordbuffer[2048];
  char neupuffer[4096];
  Cacherecverw urv(recordbuffer,2048);
  Cacherecverw def;
  Cacherecverw nurv(neupuffer,4096);
  int ok=1;         // think positiv !!!
  int err=0;
  int gef=0,dgef=0;
  int vonmeister=0;
  const char *portmsg = NULL;
  UINT4 dienstnr;
  CRDienst *dienst,*cldienst,*tocheck;
  const Clientinfo *clinfo;
  struct sockaddr_in realab;
  Zusenden anreal(puffer,2048);
  Empfangen vonreal(realab,puffer+2048,2048);
  time_t jetzt;

  // Client
  user[0]='\0';
  realm[0]='\0';
  
  ur=msg.finde(PW_USER_NAME);
  // Prfung wurde schon gemacht, brauche nur user nochmal
  getuserandrealm(user,realm,ur);
  

  clinfo = clients.finde(msg.getabsender().sin_addr,name());
  if(clinfo == NULL) 
  {
      
      if(debugflags) cerr << "Unbekannter Client " << endl;
      syslog(LOG_INFO,"Unbekannter Client %s fr Realm %s",
	     iptostr(clearpwd,msg.getabsender().sin_addr),name());
      return RJ_UNKNOWN_CLIENT;
  }
  msg.setsecret(clinfo->secret);
  msg.calc_digest(clinfo->secret);
  def.setbuffer(clinfo->defattr);

  clearpwd[0]='\0';
  dienstnr = 0;

  random_vector(csi.vector);
  anreal.initsend(PW_AUTHENTICATION_REQUEST|PW_RW_PROXY_BIT,(u_char)seqnr(clc.seqfile),csi.vector);
 
  if(realm[0]=='\0')
  {
    char platz[256];
    snprintf(platz,256,"%s@%s",user,name());
    anreal.addattr(PW_USER_NAME,platz,strlen(platz));
  }
  else
  {
    anreal.addattr(PW_USER_NAME,(char *)(ur->data),ur->len-2);
  }
  anreal.calc_digest(csi.secret);
  
  /*-- Aufbereitung der Anforderung und zusammenstellung der cache-Anfrage --*/
  for(auth_attr *akt = msg.first();akt !=NULL;akt = msg.next())
  {
      switch(akt->id)
      {
      case PW_USER_NAME:
	   break; // siehe oben
      case PW_PASSWORD:
          {
	    int passlen=akt->len-2;
	    u_char cryptpw[passlen+1];
	    ciffrepassword((u_char *)clearpwd,akt->data,akt->len-2,
			   msg.getdigest());
	    ciffrepassword(cryptpw,(u_char *)clearpwd,passlen,
			   anreal.getdigest());
	    anreal.addattr(PW_PASSWORD,(char *)cryptpw,passlen);
	  }
	  authpwd=akt;
	  break;
      case PW_CHAP_PASSWORD:
	  // Wie cache ich PW_CHAP_PASSWORD ?
	  anreal.addattr(PW_CHAP_PASSWORD,(char *)(akt->data),akt->len);
	  authpwd=akt;
	  break;
      case PW_USER_SERVICE_TYPE:	  
	  mergeservice(dienstnr,getintval(akt));
	  anreal.addattr(PW_USER_SERVICE_TYPE,(char *)(akt->data),akt->len);
	  break;
      case PW_FRAMED_PROTOCOL:
	  mergeproto(dienstnr,getintval(akt));
	  anreal.addattr(PW_FRAMED_PROTOCOL,(char *)(akt->data),akt->len);
	  break;
      default:
	  anreal.addattr(akt->id,(char *)(akt->data),akt->len);
	  break;
      }
  }


  if(authpwd == NULL)
  {
    syslog(LOG_ERR,"No Password from %s\n",iptostr(clearpwd,msg.getabsender().sin_addr));
    if(debugflags) 
    {
      cerr << "No Password from " << clearpwd << endl;
    } 
    return RJ_NO_PASSWD;
  }
  if(debugflags & 2)  
    cout << "\nUser: " << user << " Realm: " << realm << '\n'
	 << " Pawort: " << clearpwd << "  "
	 << (authpwd->id == PW_CHAP_PASSWORD ? "CHAP" : "PAP")
	 << "\n\n";
 
  if(dienstnr == 0)
  {
    syslog(LOG_ERR,"Kein Dienst von %s\n",iptostr(clearpwd,msg.getabsender().sin_addr));
    if(debugflags) 
    {
      cerr << "Kein Dienst " << clearpwd << endl;
    } 
    return RJ_NO_SERVICE;
  }

  try
  {
    if(db.lese(user,strlen(user),&urv)>=0)
    {
      gef=1;
      int passck = 0;
      if(debugflags)
      {
	cout << "User:" << user << " ";
	urv.print(user,cout);
      }
      
      tocheck = urv.findedienst(0); // Check-Attribute
      if(tocheck == NULL)
      {
	syslog(LOG_ERR,"Fr User %s@%s sind keine Checkattribute gespeichert",
	       user,realm);
	ok=0;
	gef=0;dgef=0; // Ohne Checkattribute ist wie nicht gefunden
      }
      for(auth_attr *akt = urv.first(tocheck);akt!=NULL && ok == 1;akt = urv.next())
      {
	switch(akt->id)
	{
	case PW_PASSWORD:
	  {
	    char pwt = ( memcmp(UNIX_PWD,akt->data,akt->len-2)==0 ? 'U' : 'C');
	    ok = ok && auth_pass(portmsg,authpwd,clearpwd,pwt,akt->data,(int)(akt->len)-2,msg);
	    if(!ok)
	    {
	      if(portmsg == NULL) portmsg = "Pawort falsch";
	      if(debugflags) { cerr << "Pawort falsch\n";}
	      err=RJ_WRONG_PASSWD;
	    }
	    passck = 1;
	  }
	  break;
	case ASCEND_PW_EXPIRATION:
	  {
	    struct timeval tv;
	    long expt = (long)ntohl(*((UINT4 *)(akt->data)));
	    gettimeofday(&tv,NULL);
	    if(expt > tv.tv_sec+604800)  // eine Woche
	    {
	      ok = ok && 1;
	    }
	    else if(expt > tv.tv_sec)
	    {
	      ok = ok && 1;
	      if(!portmsg) portmsg = "Pawortzeit < 7 Tage";
	    }
	    else
	    {
	      ok = 0;
	      if(!portmsg) portmsg = "Pawort abgelaufen\n";
	      if(debugflags) { cerr << "Pawort abgelaufen\n";}
	      err = RJ_USER_TIMEOUT;
	    }
	  }
	  break;
	default:
	  {
	    auth_attr *ck;
	    for(ck = msg.first();ck!=NULL && akt->id != ck->id;ck=msg.next());
	    ok = (ck != NULL && akt->len == ck->len &&  
		  memcmp((char *)(akt->data),(char *)(ck->data),ck->len-2)==0);
	    if(!ok) err=RJ_CHECK_DONT_MATCH;
	    if(debugflags)
	    {
	      if(ck==NULL) cerr << "Attribut " << dict[akt->id] << " nicht vorhanden \n";
	      else if(akt->len != ck->len || memcmp((char *)(akt->data),(char *)(ck->data),ck->len-2)!=0)
		cerr << "Attribut " << dict[akt->id] << " falsch!\n";
	    }
	  }
	}
      } /* for ... */
      if(!passck)
      {
	ok = 0;
	err=RJ_NO_PASSWD;
	if(debugflags) cerr << "Kein Pawort geprft !\n";
      }
    }
    else
    {
      if(debugflags) cout << "Nutzer " << user << " nicht gefunden ! \n";
      portmsg = "Nutzer nicht gefunden !";
      ok = 0;
      err=RJ_USER_NOT_FOUND;
    }
  }
  catch(DbException &dbe)
  {
    syslog(LOG_ERR,"db-Fehler: %s",dbe.what());
    ok=0;
    err=RJ_INTERNAL_ERROR;
  }

  // Bietet Client Dienst an ?
  cldienst = def.findedienst(dienstnr);
  if(cldienst == NULL)
  {
    portmsg = "Dienst wird nicht angeboten\n";
    ok=0;
    err=RJ_REALM_NO_SERVICE;
  }
  
  // Darf User dienst verwenden ?
  dgef = gef;
  if(ok)
  {
    dienst = urv.findedienst(dienstnr);
    if(dienst == NULL)
    {
      portmsg = "Keine Berechtigung fr angeforderten Dienst";
      ok=0;
      dgef=0;  // Nutzer und Dienst nicht gefunden
    }
  }
  
  if((ok==0 || (dgef && time(&jetzt)-dienst->czeit > cachetimeout))
     && (niemandneues==0 || gef==1)
     && cldienst != NULL
    )
  {
    if(debugflags) { cerr << "Frage den anderen Radiusserver ..." << endl; }
    vonmeister=1;
    if(komuwithserver(clsock,anreal,vonreal,clc.radhosts,csi,clc))
    {
      ok = ( vonreal.code() == PW_AUTHENTICATION_ACK );
      portmsg=NULL;
      if(debugflags) { cerr << "Antwort:" << endl; vonreal.print(cerr); }
    }
    else
    {
      if(debugflags) cerr << "Anfrage ist Fehlgeschlagen \n";
      logmsg(LOG_WARNING,"Anfrage an Meister ist fehlgeschlagen");
      vonmeister=0;
      err=RJ_INTERNAL_ERROR;
    }
  }
  if(ok)
  {
      if(debugflags) cout << "Anforderung angenommen!\n";
      antwort.initsend(PW_AUTHENTICATION_ACK,msg.id(),msg.getvector());
      
      if((msg.code() & PW_RW_PROXY_BIT)==0)
      {
	antwort.copyreplyattr(def.erstes(cldienst),def.lenreply(cldienst));
	// Merging ??? Annahme: Letze Attribute gelten immer !
      }
      if(vonmeister)
      {
	antwort.copyreplyattr(vonreal.first(),vonreal.nettolen());
      }
      else
      {
	antwort.copyreplyattr(urv.erstes(dienst),urv.lenreply(dienst));
      }
  }
  else
  {
      if(debugflags) cout << "Anforderung abgelehnt !\n";
      antwort.initsend(PW_AUTHENTICATION_REJECT,msg.id(),msg.getvector());
  }
  if(portmsg)
  {
      antwort.addattr(PW_PORT_MESSAGE,portmsg,strlen(portmsg));
  }
  antwort.calc_responsedigest(msg.getsecret());
  if(debugflags) antwort.print(cout);
  antwort.send(sockfd,msg.getabsender());
  
  // Nun ist der Kunde erstmal zufrieden und wir knnen ... 
  if(ok && vonmeister && (niemandneues==0 || gef))
  {

    if(authpwd->id == PW_PASSWORD)
    {
      if(gef)
      {
	nurv.copyreplace(PW_PASSWORD,clearpwd,strlen(clearpwd),0,&urv);
      }
      else
      {
	nurv.neuerdienst(0);
	nurv.add(PW_PASSWORD,clearpwd,strlen(clearpwd));
      }
    }
    // Fixme: How to Cache CHAP and other
      
    nurv.neuerdienst(dienstnr);
    nurv.addm(vonreal.first(),vonreal.nettolen());
    nurv.dienstcachetime(jetzt);
    
    if(gef)
    {
      for(CRDienst *c=urv.ersterdienst();c!=NULL;c=urv.weitererdienst())
      {
	if(c->dienst == dienstnr || c->dienst == 0) continue;
	nurv.copydienst(c,&urv);
      }
    }
    try 
    {
      db.schreibe(user,&nurv);
    }
    catch(DbException &dbe)
    {
      syslog(LOG_ERR,"db-Fehler: %s",dbe.what());
    }
  }
  return err;
}




