/**************************************************************
 * Modul.....: db_realm.cc                                    *
 * Zweck.....: Implementiert Klasse db_realm                  *
 * Author....: Rudolf Weber Informatikberatung Ravensburg     *
 * Copyright.: 1998 GNU                                       *
 **************************************************************/
#include<iostream.h>
#include<string.h>
#include<stdlib.h>
#include<unistd.h>
#include<errno.h>
#include<sys/time.h>
#include<ctype.h>
#include<syslog.h>
#include"radius.h"
#include"md5.h"
#include"auth.h"
#include"db_realm.h"
#include"clients.h"

extern Attrdict dict;
extern Clients  clients;
extern int debugflags;

Db_realm::~Db_realm()
{
  if(debugflags)
  {
    cerr << "~Db_realm(): Warum nur so destruktiv ..." << endl;
  }
  (void)ende();
}

Db_realm::ende()
{
  db.close();
  if(debugflags)
  {
    cerr << "Beende db_adminrealm " << name() << endl;
  }
}

int Db_realm::radauthenticate(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  Db_realm *dr=(Db_realm *)r;
  return dr->radauthenticate(sockfd,msg,antwort);
}

int Db_realm::accounting(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  return r->accounting(sockfd,msg,antwort);
}

int Db_realm::passchange(Realm *r,int sockfd,Empfangen &msg,Zusenden &antwort)
{
  Db_realm *dr=(Db_realm *)r;
  return dr->passchange(sockfd,msg,antwort);
}

int Db_realm::init(char *parameterstr)
{
  char *p;
  char *relname=parameterstr;

  initaktionen(); // hier ist sind die virtuelle tabelle fertig aufgebaut
  /* Interpretiere Parameter */
  for(p=parameterstr;*p!='\0' && isalnum(*p) && p-parameterstr<64;p++);
  if(*p!='\0') 
  { 
    throw Syntaxerror("Ungltiger Relationsname");
    return -1;
  }
  strcpy(dbname,relname);
  return 0;
}

int Db_realm::anfang()
{
  try
  {
    db.init(dbname,UDB_READ);
  }
  catch(DbException &dbe)
  {
    throw Databaseerror(dbe.what());
    return -1;
  }
  return 0;
}

void Db_realm::initaktionen()
{
  aktionen[PW_AUTHENTICATION_REQUEST]=Db_realm::radauthenticate;
  aktionen[PW_ACCOUNTING_REQUEST]=Db_realm::accounting;
  aktionen[PW_PASSWORD_REQUEST]=Db_realm::passchange;
#ifdef RADADMINCOMPAT
  aktionen[PW_RW_NEWUSER_REQUEST]=Db_realm::adminrequest;
  aktionen[PW_RW_NEWUSER_REJECT]=Db_realm::adminrequest;
  aktionen[PW_RW_USERUPD_REQUEST]=Db_realm::adminrequest;
#endif
}

int Db_realm::radauthenticate(int sockfd,Empfangen &msg,Zusenden &antwort)
{
  extern int debugflags;
  char user[256];
  char realm[256];
  char clearpwd[256];
  auth_attr *authpwd=NULL; // Zeiger auf das Passwort
  auth_attr *ur;
  char recordbuffer[2048];
  Cacherecverw urv(recordbuffer,2048);
  Cacherecverw def;
  int ok=1;         // think positiv !!!
  int err=0;
  const char *portmsg = NULL;
  UINT4 dienstnr;
  CRDienst *dienst,*cldienst;
  const Clientinfo *clinfo;

  user[0]='\0';
  realm[0]='\0';
  
  ur=msg.finde(PW_USER_NAME);
  // Prfung wurde schon gemacht, brauche nur user nochmal
  getuserandrealm(user,realm,ur);


  clinfo = clients.finde(msg.getabsender().sin_addr,name());
  if(clinfo == NULL) 
  {
      
      if(debugflags) cerr << "Unbekannter Client " << endl;
      syslog(LOG_INFO,"Unbekannte Clientaddresse %s fr realm %s",
	     iptostr(clearpwd,msg.getabsender().sin_addr),name());
      return RJ_UNKNOWN_CLIENT;
  }
  msg.setsecret(clinfo->secret);
  msg.calc_digest(clinfo->secret);
  def.setbuffer(clinfo->defattr);

  clearpwd[0]='\0';
  dienstnr = 0;
  for(auth_attr *akt = msg.first();akt !=NULL;akt = msg.next())
  {
      switch(akt->id)
      {
      case PW_PASSWORD:
	  ciffrepassword((u_char *)clearpwd,akt->data,akt->len-2,msg.getdigest());
	  /* fall through */
      case PW_CHAP_PASSWORD:
	  authpwd=akt;
	  break;
      case PW_USER_SERVICE_TYPE:	  
	  mergeservice(dienstnr,getintval(akt));
	  break;
      case PW_FRAMED_PROTOCOL:
	  mergeproto(dienstnr,getintval(akt));
	  break;
      default:
	  break;
      }
  }


  if(authpwd == NULL)
  {
    syslog(LOG_ERR,"No Password from %s\n",iptostr(clearpwd,msg.getabsender().sin_addr));
    if(debugflags) 
    {
      cerr << "No Password from " << clearpwd << endl;
    } 
    return  RJ_NO_PASSWD;
  }
  if(debugflags & 2)  
    cout << "\nUser: " << user << " Realm: " << realm << '\n'
	 << " Pawort: " << clearpwd << "  "
	 << (authpwd->id == PW_CHAP_PASSWORD ? "CHAP" : "PAP")
	 << "\n\n";
 
  if(dienstnr == 0)
  {
    syslog(LOG_ERR,"Kein Dienst von %s\n",iptostr(clearpwd,msg.getabsender().sin_addr));
    if(debugflags) 
    {
      cerr << "Kein Dienst " << clearpwd << endl;
    } 
    return RJ_NO_SERVICE;
  }

  try
  {
    if(db.lese(user,strlen(user),&urv)>=0)
    {
      int passck = 0;
      if(debugflags)
      {
	cout << "User:" << user << " ";
	urv.print(user,cout);
      }
      
      dienst = urv.findedienst(0); // Check-Attribute
      if(dienst == NULL)
      {
	syslog(LOG_ERR,"Fr User %s@%s sind keine Checkattribute gespreichert",
	       user,realm);
	ok=0;
      }

      if(hauptdienst(dienstnr)!= PW_DIALOUT_FRAMED_USER)
      {
	for(auth_attr *akt = urv.first(dienst);akt!=NULL && ok == 1;akt = urv.next())
        {
	  switch(akt->id)
	  {
	  case PW_PASSWORD:
	    {
	      char pwt = ( memcmp(UNIX_PWD,akt->data,akt->len-2)==0 ? 'U' : 'C');
	      ok = ok && auth_pass(portmsg,authpwd,clearpwd,pwt,akt->data,(int)(akt->len)-2,msg);
	      if(!ok)
	      {
		if(portmsg == NULL) portmsg = "Pawort falsch";
		if(debugflags) { cerr << "Pawort falsch\n";}
		err=RJ_WRONG_PASSWD;
	      }
	      passck = 1;
	    }
	    break;
	  case ASCEND_PW_EXPIRATION:
	    {
	      struct timeval tv;
	      long expt = (long)ntohl(*((UINT4 *)(akt->data)));
	      gettimeofday(&tv,NULL);
	      if(expt > tv.tv_sec+604800)  // eine Woche
	      {
		  ok = ok && 1;
	      }
	      else if(expt > tv.tv_sec)
	      {
		ok = ok && 1;
		if(!portmsg) portmsg = "Pawortzeit < 7 Tage";
	      }
	      else
	      {
		ok = 0;
		if(!portmsg) portmsg = "Pawort abgelaufen\n";
		if(debugflags) { cerr << "Pawort abgelaufen\n";}
		err=RJ_USER_TIMEOUT;
	      }
	    }
	    break;
	  default:
	    {
	      auth_attr *ck;
	      for(ck = msg.first();ck!=NULL && akt->id != ck->id;ck=msg.next());
	      ok = (ck != NULL && akt->len == ck->len &&  
		  memcmp((char *)(akt->data),(char *)(ck->data),ck->len-2)==0);
	      if(!ok) err = RJ_CHECK_DONT_MATCH;
	      if(debugflags)
	      {
		if(ck==NULL) cerr << "Attribut " << dict[akt->id] << " nicht vorhanden \n";
		else if(akt->len != ck->len || memcmp((char *)(akt->data),(char *)(ck->data),ck->len-2)!=0)
		  cerr << "Attribut " << dict[akt->id] << " falsch!\n";
	      }
	    }
	  }
	} /* for ... */
	if(!passck)
	{
	  ok = 0;
	  if(debugflags) cerr << "Kein Pawort bergeben !\n";
	  err=RJ_NO_PASSWD;
	}
      } /* if ! dialout-User */
      else /* Dial-out-User - prfe nur Time-Out */
      {
	auth_attr *akt;
	urv.first(dienst);
	if((akt=urv.finde(ASCEND_PW_EXPIRATION))!=NULL)
	{
	  struct timeval tv;
	  long expt = (long)ntohl(*((UINT4 *)(akt->data)));
	  gettimeofday(&tv,NULL);
	
	  if(expt > tv.tv_sec)
	  {
	    ok = ok && 1;
	  }
	  else
	  {
	    ok = 0;
	    if(debugflags) { cerr << "Pawort abgelaufen\n";}
	    err=RJ_USER_TIMEOUT;
	  }
	}
      }
    }
    else
    {
      if(debugflags) cout << "Nutzer " << user << " nicht gefunden ! \n";
      portmsg = "Nutzer nicht gefunden !";
      ok = 0;
      err=RJ_USER_NOT_FOUND;
    }
  }
  catch(DbException &dbe)
  {
    syslog(LOG_ERR,"db-Fehler: %s",dbe.what());
    ok=0;
    err=RJ_INTERNAL_ERROR;
  }

  // Bietet Client Dienst an ?
  if(ok)
  {
    cldienst = def.findedienst(dienstnr);
    if(cldienst == NULL)
    {
      portmsg = "Dienst wird nicht angeboten\n";
      ok=0;
      err=RJ_REALM_NO_SERVICE;
    }
  }
  // Darf User dienst verwenden ?
  if(ok)
  {
    dienst = urv.findedienst(dienstnr);
    if(dienst == NULL)
    {
      portmsg = "Keine Berechtigung fr angeforderten Dienst";
      ok=0;
      err=RJ_USER_NO_SERVICE;
    }
  }
  
  if(ok)
  {
      if(debugflags) cout << "Anforderung angenommen!\n";
      antwort.initsend(PW_AUTHENTICATION_ACK,msg.id(),msg.getvector());
      
      if((msg.code() & PW_RW_PROXY_BIT)==0)
      {
	antwort.copyreplyattr(def.erstes(cldienst),def.lenreply(cldienst));
	// Merging ??? Annahme: Letze Attribute gelten immer !
      }
      antwort.copyreplyattr(urv.erstes(dienst),urv.lenreply(dienst));
  }
  else
  {
      if(debugflags) cout << "Anforderung abgelehnt !\n";
      antwort.initsend(PW_AUTHENTICATION_REJECT,msg.id(),msg.getvector());
  }
  if(portmsg)
  {
      antwort.addattr(PW_PORT_MESSAGE,portmsg,strlen(portmsg));
  }
  antwort.calc_responsedigest(msg.getsecret());
  if(debugflags) antwort.print(cout);
  antwort.send(sockfd,msg.getabsender());

  return err;
}

int Db_realm::passchange(int sockfd,Empfangen &msg,Zusenden &antwort)
{
  extern int debugflags;
  char user[256];
  char realm[256];
  char errmsg[128];
  char altpuffer[4096];
  char neubuffer[4096];
  char clearpwdalt[AUTH_PASS_LEN];
  char clearpwdneu[AUTH_PASS_LEN];
  char clearpwddat[AUTH_PASS_LEN];
  auth_attr *oldpwattr = NULL;
  auth_attr *newpwattr = NULL;
  Cacherecverw urv(altpuffer,4096);
  Cacherecverw neu(neubuffer,4096);
  
  int ok=1;         // think positiv !!!
  char *portmsg = NULL;
  user[0]='\0';

  /* Aufbereitung der Anforderung */
  for(auth_attr *akt = msg.first();akt !=NULL;akt = msg.next())
  {
      switch(akt->id)
      {
      case PW_USER_NAME:
	  getuserandrealm(user,realm,akt);
	  break;
      case PW_PASSWORD: // neues Passwort
          newpwattr=akt;
	  break;
      case PW_OLD_PASSWORD: // altes Passwort
	  oldpwattr = akt;
      default:
	  break;
      }
  }
  if(user[0]=='\0' || newpwattr == NULL || oldpwattr == NULL)
  {
    if(debugflags)
    {
      cerr << "Nachricht unvollkommen !\n";
    }
    return -1;
  }
  
  try
  {
    if(db.lese(user,strlen(user),&urv)>=0)
    {
      int passck = 0;
      auth_attr *datpwd;
      if(debugflags)
      {
	cout << "User:" << user << " Realm:" << realm << " ";
	urv.print(user,cout);
      }
      if(urv.findedienst(0)==NULL
	 || (datpwd = urv.finde(PW_PASSWORD))==NULL)
      {
	syslog(LOG_ERR,"User %s@%s hat kein Passwort !\n",user,realm);
	return -1;
      }
   
      getstring(clearpwddat,datpwd);
      msg.calc_digest((const u_char *)clearpwddat);
      ciffrepassword(clearpwdalt,oldpwattr->data,oldpwattr->len-2,
		                                              msg.getdigest());
      ciffrepassword(clearpwdneu,newpwattr->data,newpwattr->len-2,
                                                              msg.getdigest());
      if(debugflags & 2)
      {
	cerr << "Altes Pwd " << clearpwdalt 
	     << " gespeichertes Pwd " << clearpwddat 
             << " neues Passwort " << clearpwdneu << "\n";
      }
      // Unix-Passwort ?
      if(strcmp(UNIX_PWD,clearpwddat)==0)
      {
	const char *ret = unix_pass(user,clearpwdalt);
	ok = ok && (ret == NULL);
	portmsg = ret;
      }
      else
      {
	ok=ok && (strcmp(clearpwddat,clearpwdalt)==0);
      }
      if(!ok)
      {
	if(portmsg == NULL) portmsg = "Pawort falsch";
	if(debugflags) { cerr << "Pawort falsch\n";}
      }
      
      if(ok)
      {
	neu.copyreplace(PW_PASSWORD,clearpwdneu,strlen(clearpwdneu),0,&urv);
	for(CRDienst *ad=urv.ersterdienst();ad!=NULL;ad=urv.weitererdienst())
	{
	  if(ad->dienst==0) continue;
	  neu.copydienst(ad,&urv);
	}
        db.schreibe(user,&neu);
        sprintf(errmsg,"Passwort von Nutzer %s gendert",user);
	portmsg = errmsg;
	if(debugflags)
        {
	  cout << portmsg << "\nDer neue Datensatz ist:\n";
          neu.print(user,cout);
	  cout << endl;
	}
      }
      else
      {
	portmsg = "Altes Passwort falsch !";
	if(debugflags) cout << portmsg << endl;
      }
    }
    else
    {
      ok=0;
      sprintf(errmsg,"Nutzer %s nicht vorhanden !",user);
      portmsg = errmsg;
      if(debugflags)  cerr << portmsg << endl;
    }
  }
  catch(DbException &dbe)
  {
    syslog(LOG_ERR,"db-Fehler: %s",dbe.what());
    sprintf(errmsg,"Interner DB_Fehler %s",dbe.what());
    portmsg = errmsg;
    ok=0;
  }
  /* senden der Antwort */
  if(ok)
  {
      if(debugflags) cout << "Pawortnderung erfolgreich !\n";
      antwort.initsend(PW_PASSWORD_ACK,msg.id(),msg.getvector());
  }
  else
  {
      if(debugflags) cout << "Pawortnderung abgelehnt !\n";
      antwort.initsend(PW_PASSWORD_REJECT,msg.id(),msg.getvector());
  }
  if(portmsg)
  {
      antwort.addattr(PW_PORT_MESSAGE,portmsg,strlen(portmsg));
  }
  antwort.calc_responsedigest((const u_char *)clearpwdalt);
  if(debugflags) antwort.print(cout);
  antwort.send(sockfd,msg.getabsender());
}




