/***********************************************************************
 * falle.cc...: Falle fr Verbrecher                                   *
 * Author.....:  Rudolf Weber Informatikberatung Ravensburg            *
 * Copyright.....: 1997 GNU Copyleft                                   *
 ***********************************************************************/
#include<iostream.h>
#include<stdlib.h>
#include<unistd.h>
#include<sys/types.h>
#include<signal.h>
#include<sys/wait.h>
#include"conf.h"
#include"config.h"
#include"falle.h"
#include"ipaddrtools.h"
extern int debugflags;

static void fatalfunc(char *errmsg)
{
  cerr << "gdbmerror: " << errmsg << endl;
  exit(1);
}

int Falle::init(char *datname,int flags)
{
  strcpy(dateiname,datname);
  Db::open(datname,DB_HASH,flags,0640,NULL,(DbInfo *)0,&fdb);
}


int Falle::schreibe(char *name,char *script)
{
  Dbt key(name,strlen(name));
  Dbt data(script,strlen(script));

  fdb->put(0,&key,&data,0);
}

int Falle::lese(const char *name,int nlen,char *script,int slen)
{
  int ret;
  Dbt key(name,nlen);
  Dbt data;
  data.set_flags(DB_DBT_USERMEM);data.set_data(script);data.set_ulen(slen);

  ret=fdb->get(0,&key,&data,0);
  if(ret==DB_NOTFOUND)
  {
    script[0]='0';
    return -1;
  }
  script[data.get_size()]='\0';
  return 0;
}

int Falle::aktion(const char *name,int nlen,UINT4 ip,UINT4 event)
{
  char script[80];
  char ipstr[16];
  pid_t pid;
  extern char **environ;

  if(lese(name,nlen,script,80)==0)
  {
    if(debugflags)
    {
      cerr << "Falle bei " << name << " zugeschnappt \n";
    }
    if((pid = fork())>0) return 1; // Vater
    else if(pid == 0) 
    { // Sohn
      char scriptpath[256];
      const char *arg[5];
      struct in_addr ipaddr;
      if(script[0]=='/') strcpy(scriptpath,script);
      else              snprintf(scriptpath,256,"%s/%s",RADFALLESCRIPTDIR,script);
      if(debugflags)
	{
	  cerr << "Starte " << scriptpath << endl;
	}
	
      ipaddr.s_addr = ip;
      iptostr(ipstr,ipaddr);
      arg[0]=script;
      arg[1]=name;
      arg[2]=ipstr;
      arg[3]=(event == 1 ? "Start" : "Stop");
      arg[4]=NULL;
      execve(scriptpath,arg,environ);
    }
  }
}

int Falle::loesche(char *name)
{
  int ret;
  Dbt key(name,strlen(name));
  ret = fdb->del(0,&key,0);
  return ret;
}

static void sigchldhandler(int sig)
{
  int status;
  pid_t pid;

  pid = wait(&status);
  if(debugflags)
  {
    cerr << "Kindproze " << pid << " wurde beendet\n";
  }
}

void install_sigchldhandler()
{
  struct sigaction act,oact;
  sigset_t sigset;

  sigemptyset(&sigset);
  act.sa_handler = sigchldhandler;
  act.sa_mask = sigset;
  act.sa_flags = SA_RESTART;
  sigaction(SIGCHLD,&act,&oact);
}

#ifdef FALLE_HAUPT

int debugflags = 1;

void usage()
{
  cerr << "Usage: radfalle add <name> <script>\n\tradfalle name\n"
       << "\tradfalle del <name>\n\tradfalle schnapp <name> <ip> [Start|Stop]\n";

}

void main(int argc,char **argv)
{
  Falle falle;
  if(argc < 2) 
  {
    usage();
    exit(1);
  }
  
  install_sigchldhandler();
  falle.init(RADFALLEPATH,FF_CREATE);
  if(strcmp(argv[1],"add")==0)
  {
    if(argc<4) { usage(); exit(1); }
    falle.schreibe(argv[2],argv[3]);
  }
  else if(strcmp(argv[1],"del")==0)
  {
    if(argc<3) { usage(); exit(1); }
    falle.loesche(argv[2]);
  }
  else if(strcmp(argv[1],"schnapp")==0)
  {
    struct in_addr ip;
    if(argc<5) { usage(); exit(1); }
    parseipaddr(ip,argv[3]);
    falle.aktion(argv[2],strlen(argv[2]),(UINT4)ip.s_addr,(strcmp(argv[4],"Start")==0 ? (u_char)1 : (u_char)2));
  }
  else if(strcmp(argv[1],"list")==0)
  {
    for(Falleit it(falle);it()!=NULL;it.next())
    {
      cout << it() << "  " << it.script() << endl;
    }
  }
}
#endif




