/**************************************************************************
 * Rudolfs Radius Demon                                                   *
 * Nun schlu mit der Bettnsserei - ich baue meinen eigenen Daemon       *
 *                                                                        *
 * copyright: 1996,1997,1998: Rudolf Weber Ravensburg                     *
 * email: rw@weber.rv.infnet.verein.de                                    *
 * Rights: see GNU-Copyleft (Distribution only with Source-Code)          *
 * Comericial Users are ask to send money                                 *
 **************************************************************************/
#include<iostream.h>
#include<string.h>
#include<stdlib.h>
#include<unistd.h>
#include<errno.h>
#include<sys/time.h>
#include<sys/types.h>
#include<sys/socket.h>
#include<sys/file.h>
#include<netinet/in.h>
#include<getopt.h>
#include<signal.h>
#include<syslog.h>
#include"logmsg.h"
#include"opensocket.h"
#include"radius.h"
#include"clients.h"
#include"realms.h"
#include"radlog.h"
#include"falle.h"
#include"sysvmp/mlsverw_allg.h"
#include"config.h"
#include"debug.h"

struct rfeldelement : feldelement
{
  int antwfd;
  struct sockaddr_in absender;
  int                lang;
  struct auth_hdr    empfangen;
};

struct ServKontext
{
  int sockfd;
  int accfd;

  ServKontext() { sockfd=-1;accfd=-1; }
};

inline int rfeldplatz()
{
  return getpagesize()-sizeof(rfeldelement)+sizeof(auth_hdr);
}

char *schreibe_auftragsnr(char *puffer,rfeldelement &f)
{
  char s[16];
  char *p=puffer;
  sprintf(puffer,"%s:10 %d",iptostr(s,f.absender.sin_addr),
	  ntohs(f.absender.sin_port),(int)f.empfangen.id);

  return puffer;
}

int vglauftragsnr(const rfeldelement& e1,const rfeldelement& e2)
{
  return e1.absender.sin_addr.s_addr == e2.absender.sin_addr.s_addr
    &&   e1.absender.sin_port == e2.absender.sin_port
    &&   e1.empfangen.id == e2.empfangen.id;
}

extern char *progname;
int debugflags=0;
int fang=0;  
char *progname;
pid_t pgrp;
int anzarbeiter=1;
Attrdict dict;
Clients clients;
Realms  realms;  // Liste mit User-hash-tabellen
Logspez logsp;
FieldLogfile log;
PaketLogfile paketlog;
PaketLoglevel pll;
int           plaktiv=0;
Falle   falle;

int empfange_auftrag(rfeldelement &f,void *vp)
{
  int sr;
  fd_set readfds; 
  int salen = sizeof(struct sockaddr_in);
  ServKontext *sv = (ServKontext *)vp;
  FD_ZERO(&readfds);
  FD_SET(sv->sockfd,&readfds);
  FD_SET(sv->accfd,&readfds);
  sr = select(sv->accfd+1,&readfds,NULL,NULL,NULL);
  if(sr < 0)
  {
    if(errno == EINTR) return -1;
    cerr << "main:select: " << strerror(errno) << endl;
    exit(1);
  }
  if(FD_ISSET(sv->sockfd,&readfds)) f.antwfd=sv->sockfd;
  else if(FD_ISSET(sv->accfd,&readfds)) f.antwfd=sv->accfd;
  else return -1; // Was ist los ???
  
  f.lang = recvfrom(f.antwfd,(char *)&(f.empfangen),rfeldplatz(),0,(struct sockaddr *)&(f.absender),&salen);

  if(f.lang <=0) return -1;
  if(debugflags)
  {
    cout << "\n\nAnforderung: ";
    Empfangen authanf(f.absender,(char *)&(f.empfangen),f.lang);
    authanf.print(cout);
  }
  return 0;
}

// -1 : Fehler
int bearbeite_auftrag(rfeldelement &f)
{
  static char puffer[64];
  char user[256];
  char realm[256];
  auth_attr *userattr;
  Realm *rp;
  Raktion fktp;
  int r;
  
  Empfangen authanf(f.absender,(char *)&(f.empfangen),f.lang);
  Zusenden  replymsg((char *)&(f.empfangen)+f.lang,rfeldplatz()-f.lang);
  f.stat=S_lokalarbeit;
 
  if(debugflags)
  {
    authanf.print(cout);
  }
  if(plaktiv && ( pll == PLL_ALL || authanf.code()==PW_AUTHENTICATION_REQUEST  && pll == PLL_AUTH))
  {
    paketlog.log(authanf);
  }
  // Bestimme realm
  user[0]='\0';realm[0]='\0';
  if((userattr=authanf.finde(PW_USER_NAME))!=NULL)
  {
    getuserandrealm(user,realm,userattr);
    if((rp=realms[realm])==NULL)
    {
      // leider nichts zu machen ....
      syslog(LOG_ERR,"Realm %s gibt es bei uns nicht ...",realm);
      if(plaktiv && pll == PLL_WRONGPW)
      {
         paketlog.logstr("#Unbekannter Realm %s\n",realm);
         paketlog.log(authanf);
      } 
      return -1;
    }
  }
  else
  {
    // Anforderung ohne User kommt an adminrealm
    rp =realms.getadminrealm();
  }
  fktp = rp->aktion(authanf.code());
  if(fktp == NULL)
  {
    char kleinerplatz[32];
    const char *codestr;
    if((codestr=msgtype2str(authanf.code()))==NULL)
    {
      snprintf(kleinerplatz,32,"%d",(int)(authanf.code()));
      codestr = kleinerplatz;
    }
    syslog(LOG_ERR,"Realm %s bearbeitet %s nicht",rp->name(),codestr);
    if(plaktiv && pll == PLL_WRONGPW)
    {
      paketlog.logstr("#Realm %s kann folgendes Paket nicht bearbeiten:\n",rp->name);
      paketlog.log(authanf);
    } 
    return -1;
  }
  r=(*fktp)(rp,f.antwfd,authanf,replymsg);
  if(r<0 && plaktiv && pll == PLL_WRONGPW)
  {
    switch(r)
    {
    case RJ_UNKNOWN_CLIENT:
      paketlog.logstr("#unbekannter Absender\n");
      break;
    case RJ_NO_PASSWD:
      paketlog.logstr("#Paket ohne Passwd\n");
      break;
    case RJ_NO_SERVICE:
      paketlog.logstr("#Paket ohne User-Service\n");
      break;
    case RJ_WRONG_PASSWD: return r; // macht schon auth (mit passwd)
    case RJ_USER_TIMEOUT:
      paketlog.logstr("#Die Zeit des Nutzers ist abgelaufen\n");
      break;
    case RJ_USER_NOT_FOUND:
      paketlog.logstr("#Nutzer nicht gefunden\n");
      break;
    case RJ_REALM_NO_SERVICE:
      paketlog.logstr("#Der Realm bietet den Dienst nicht an \n");
      break;
    case RJ_USER_NO_SERVICE:
      paketlog.logstr("#Der Nutzer darf den Dienst nicht verwenden \n");
      break;
    case RJ_CHECK_DONT_MATCH:
      paketlog.logstr("# Ein check-attribut pat nicht \n");
    case RJ_INTERNAL_ERROR:
    default:
      paketlog.logstr("Ein Interner Fehler ist aufgetreten\n"); 
    }
    paketlog.log(authanf);
  }
  return r;
}

void fertigerauftrag(rfeldelement &f,void *vp)
{
  char puffer[64];
  DEBUGCODE(1,cout << "Auftrag " << schreibe_auftragsnr(puffer,f) << "ist schon lange fertig !!!! \n";);
  Zusenden msg((char *)&(f.empfangen)+f.lang,rfeldplatz()-f.lang);

  DEBUGCODE(1,cout << "Wiederholung der Antwort:\n";msg.print(cout);
              cout << endl;);
    
  msg.send(f.antwfd,f.absender);
}

void arbeiterverstorben(rfeldelement &f,void *vp)
{
  DEBUGCODE(1,cerr << "Arbeiter " << f.apid << " muss irgendwie verstorben sein ... " << endl;);
  syslog(LOG_ERR,"Arbeiter %d muss irgendwie verstorben sein ... ",f.apid);
}


void check_procmask()
{
  int ret;
  sigset_t oldmask,newmask;
  sigemptyset(&newmask);
  sigprocmask(SIG_BLOCK,&newmask,&oldmask);

  if((ret=sigismember(&oldmask,SIGHUP))<0)
  {
    cerr << "sigismember fehlgeschlagen " << endl;
    return;
  }
  cerr << "SIGHUP " << ( sigismember(&oldmask,SIGHUP) ? "ist" : "ist nicht")
       << "blockiert" << endl;
}

static void sighuphandler(int sig)
{
  DEBUGCODE(1,cerr << "Reinitialisere Logfile ... " << endl;);
  log.reinit(); 
}

static void sighandlerplogonoff(int sig)
{
  DEBUGCODE(1,cerr << "plogonoff: ... " << endl;);
  if(plaktiv)
  {
    plaktiv=0;
    paketlog.end();
  }
  else
  {
    paketlog.begin();
    plaktiv=1;
  }
}

static void sighuphandler_meister(int sig)
{
  static struct timeval letztesmal;
  struct timeval jetzt;
  gettimeofday(&jetzt,NULL);

  if(jetzt.tv_sec - letztesmal.tv_sec > 10 )
  {
    DEBUGCODE(1,cerr << "Kinder: HUP!" << endl;check_procmask());
    kill(-pgrp,sig);
    letztesmal=jetzt;
  }
}

void install_sighuphandler(int sig,void (*sa_handler)(int))
{
  struct sigaction act, oact;
  sigset_t s;
  
  sigemptyset(&s);
  // sigaddset(&s,SIGHUP);
  act.sa_handler = sa_handler;
  act.sa_mask = s;
  act.sa_flags = SA_RESTART;
  sigaction(sig, &act, &oact );
}

static void handler_TERM(int s)
{
  DEBUGCODE(1,cerr << "Ich sterbe ... " << endl;);
  realms.ende();
  exit(0);
}

static void handler_TERM_meister(int s)
{
  struct sigaction siga;
  siga.sa_handler = SIG_IGN;
  sigemptyset(&(siga.sa_mask));
  siga.sa_flags = 0;
  sigaction(s,&siga,NULL);

  DEBUGCODE(1,cerr << "Ich sterbe und reisse meine Kinder mit ... " << endl;);
  kill(-pgrp,s);
  DEBUGCODE(1,cerr << "Ich schliesse nun meine Datenbanken " << endl;);
  realms.ende();
  exit(0);
}

void install_termhandler(int s,void (*sa_handler)(int))
{
  struct sigaction siga;

  siga.sa_handler = sa_handler;
  sigemptyset(&(siga.sa_mask));
  siga.sa_flags = 0;
  sigaction(s,&siga,NULL);
}

#ifdef SINGLE
char rfeldbuf[4096];
#else

#include"sysvmp/mlsverw.h"
MLSverw<rfeldelement> mls(ftok(RADUSERSDBPATH,'R'),getpagesize(),QUEUE_SIZE,MAX_WORKER);
#endif

int main(int argc,char **argv)
{
  int result;
  char errmsg[1024];
  int salen;
  struct sockaddr_in salocal;
  ServKontext servk;
  
  int logd;
  struct option long_options[] =
  {   {"dict",1,0,1 },
      {"user",1,0,'u' },
      {"logmuster",1,0,'l'},
      {"falle",0,0,'t'},
      {"logfile",1,0,2},
      {"plogfile",1,0,3},
      {0,0,0,0}
  };
  int oi = 0;
  int c;
  char *logf = NULL;
  char *dictdir    = RADDICT;
  char *clientpath = RADCLIENTS;
  char *userdbpath = RADUSERSDBPATH;
  char *logmuster  = ACCOUNTINGMUSTER;
  char *logfile    = ACCOUNTINGFILE;
  char *paklogfile = PAKETLOGFILE;
  progname = argv[0];
  
  while((c=getopt_long(argc,argv,"vl:d:tm:L:",long_options,&oi))!=-1)
  {
    switch(c)
    {
    case 1:
      dictdir = optarg;
      cout << "Dict: " << dictdir << endl;
      break;
    case 'l':
      logf = optarg;
      cout << "lese aus logmuster " << logf << endl;
      break;
    case 'L': // Extra Paketlogging
      parsepaketloglevel(pll,optarg);
      plaktiv=1;
      break;
    case 2:  // Logfilename
      logfile = optarg;
      break;
    case 3:  // Paketlogfile
      paklogfile = optarg;
      break;
    case 'u':
      cout << "Userdatenbank :" << optarg << endl;
      userdbpath = optarg;
      break;
    case 'd':
      debugflags = atoi(optarg);
      break;
    case 't':
      cout << "Falle aktiviert " << endl;
      fang=1;
      break;
    case 'v':
      cout << "Rudolfs neue Version \n";
      break;
    case 'm': 
      anzarbeiter=atoi(optarg);
      cout << "Starte " << anzarbeiter << " Threads" << endl;
      break;
    default:
      cerr << "Unbekannte Option " << c << endl;
    }
  } 
 
  if(dictdir == NULL) 
  {
      cerr << "kein dictionary definiert \n";
      exit(1);
  }
  const char *ret = dict.init(dictdir);
  if(ret !=NULL) 
  {
      cerr << "dict.init:" << ret << endl;
      exit(1);
  }

  servk.sockfd = opensocket("radius","udp");
  servk.accfd  = opensocket("radacct","udp");
  
  if(debugflags)
  {      /*----- probeausgabe Adresse -------*/
      struct	sockaddr_in	salocal;
      salen = sizeof(salocal);
      if(getsockname(servk.sockfd,(struct sockaddr *)&salocal,&salen)<0)
      {
	  cerr << "getsockname :" << strerror(errno) << endl;exit(1);
      }
      cout << "Auth-Adresse " << salocal << endl;
      salen = sizeof(salocal);
      if(getsockname(servk.accfd,(struct sockaddr *)&salocal,&salen)<0)
      {
	  cerr << "getsockname :" << strerror(errno) << endl;exit(1);
      }
      cout << "Acct-Adresse " << salocal << endl;
      logmsginit(LOGMSG_SYSLOG,"radiusd",LOG_PID|LOG_PERROR,LOG_DAEMON);
  } 
  else
  {
      char pidbuffer[50];
      int pidf;
#ifndef DEBUG
      // Von Sitzung abhngen
      pid_t pid = fork();
      
      if(pid < 0)
      {
	  cerr <<  progname << ": Couldn't fork\n";
	  exit(-1);
      }
      if(pid > 0)
      {
	_exit(0); // bitte lasse meine globalen Daten ganz
      }
#endif
      for (int t = 32; t >= 3; t--)
      {
	  if(t != servk.sockfd && t!= servk.accfd) close(t);
	  
      }
      logmsginit(LOGMSG_SYSLOG,"radiusd",LOG_PID,LOG_DAEMON);
      pidf = open(PIDFILE,O_CREAT|O_WRONLY|O_TRUNC,0644);
      if(pidf>0)
      {
	sprintf(pidbuffer,"%d\n",getpid());
	write(pidf,pidbuffer,strlen(pidbuffer));
	close(pidf);
      }
      else
      {
	syslog(LOG_INFO,"Can't write %s",PIDFILE);
      }
      
     
  }

  setsid(); // eigene Session 
  if(setpgrp()<0) { syslog(LOG_ERR,"setpgrp:%m");}
  pgrp = getpgrp();
  DEBUGCODE(1,cout << "Prozegruppe " << pgrp << endl;);

  clients.init(clientpath,dict);
  realms.init(userdbpath,"realms");
  logsp.init(logmuster);    
  log.init(logsp,logfile);log.begin();
  if(pll != PLL_NONE)
  {
    paketlog.init(paklogfile);paketlog.begin();
  }
  if(fang)
  {
    falle.init(RADFALLEPATH,FF_RDONLY);
    install_sigchldhandler();
  }

#ifdef SINGLE
  rfeldelement *arbeit = (rfeldelement *)rfeldbuf;
  realms.anfang();
  DEBUGCODE(1,cerr << "SINGLE-Threaded " << endl;);

  install_sighuphandler(SIGHUP,sighuphandler);  
  install_sighuphandler(SIGUSR2,sighandlerplogonoff);
  install_termhandler(SIGTERM,handler_TERM);
  install_termhandler(SIGINT,handler_TERM);
  while(1)
  {
    if(empfange_auftrag(*arbeit,&servk)>=0)
    {
      bearbeite_auftrag(*arbeit);
    }
  }
#else
  for(int i=0;i<anzarbeiter;i++)
  {
    pid_t kind;

    if((kind = fork())==0)
    {
      realms.anfang();
      mls.clear_anleger();
      install_sighuphandler(SIGHUP,sighuphandler);
      install_sighuphandler(SIGUSR2,sighandlerplogonoff);
      install_termhandler(SIGTERM,handler_TERM);
      install_termhandler(SIGINT,handler_TERM);
      DEBUGCODE(1,cerr << "Prozegruppe " << getpgrp() << endl;);
      mls.arbeiter();
      // no return
      exit(0);
    }
    if(kind < 0) syslog(LOG_ERR,"fork: %m");
    DEBUGCODE(1,cerr << "Kind " << kind << "gestartet " << endl;);
  }
  DEBUGCODE(1,cerr << "Ich bin der Meister " << endl;);
  install_sighuphandler(SIGHUP,sighuphandler_meister);
  install_sighuphandler(SIGUSR2,sighuphandler_meister);
  install_termhandler(SIGTERM,handler_TERM_meister);
  install_termhandler(SIGINT,handler_TERM_meister);
  mls.meister(&servk);
#endif
}

















