/*****************************************************************************
 * Modul.....: radlog.cc                                                     *
 * Zweck.....: Implementierung Radiuslogfiles                                *
 * Autor.....: Rudolf Weber                                                  *
 * Copyright.: Informatik- und Netzwerkverein Ravensburg.e.V 1998 nach GNU   *
 *****************************************************************************/
#include<iostream.h>
#include<strstream.h>
#include<string.h>
#include<stdio.h>
#include<sys/time.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<sys/file.h>
#include<fcntl.h>
#include<signal.h>
#include<syslog.h>
#include"config.h"
#include"radlog.h"

/*----------- Hilfsklasse Logpuffer -------------------*/
Logpuffer::init()
{
  memset(puffer,'\0',1024);
  lang=0;
}

Logpuffer::schreib(int fd)
{
  int rest=lang;
  int nw=0;
  char *p=puffer;
  while(rest > 0 && nw>=0)
  {
    nw=write(fd,p,rest);
    p+=nw;
    rest-=nw;
  }
  if(nw<0) { syslog(LOG_ERR,"logfile-write %m\n"); }
}

Logpuffer::ende()
{
  lang=strlen(puffer);
  puffer[lang]='\0';
}

/*----------- Allgemeine Operationen fr logfiles -----*/
Logfile::begin()
{
  fd=open(logfilename,O_WRONLY|O_CREAT|O_APPEND,0644);
 
  if(fd<0)
  {
    syslog(LOG_ERR,"Fehler beim ffnen von %s:%m",logfilename);
    exit(1);
  }
  return 0;
}

Logfile::end()
{
  close(fd);
}

Logfile::lock()
{
  sigemptyset(&sigmask);
  sigaddset(&sigmask,SIGHUP);
  sigprocmask(SIG_BLOCK,&sigmask,&oldsigmask);
  flock(fd,LOCK_EX);
  lseek(fd,0,SEEK_END); // ein anderer knnte inzwischen gelogt haben ...
}

Logfile::unlock()
{
  flock(fd,LOCK_UN);
  sigprocmask(SIG_SETMASK,&oldsigmask,NULL);
}

Logfile::logstr(const char *format,...)
{
  va_list ap;

  va_start(ap,format);
  vsnprintf(lp.puffer,lp.maxlen(),format,ap);
  va_end(ap);
  lp.ende();
  lock();
  lp.schreib(fd);
  unlock();
}

/*-------------- Field Log -------------------------*/
int FieldLogfile::init(Logspez &ls,char *datname)
{
  Logfile::init(datname);
  spez = &ls;
}

int FieldLogfile::log(Nachricht& msg)
{
  auth_attr *aktat = msg.first();
  int erstes = 1;
  lp.init();
  ostrstream str(lp.puffer,lp.maxlen());
  
  for(Attreintrag *akt=spez->first();akt != NULL;akt=spez->next())
  { 
      if(!erstes) str << ';';
      erstes = 0;
      
      if(akt->id == PW_RW_CURRENT_TIME)
      {
	  struct timeval tv;
          time_t tsec;
	  char zstr[40];
	  gettimeofday(&tv,NULL);
          tsec = tv.tv_sec;
	  strftime(zstr,40,"%a %d.%m.%Y %H:%M:%S ",localtime(&tsec));
	  str << zstr;
      }
      else
      {
	  aktat = msg.finde(akt->id);	  
	  if(aktat!=NULL) akt->print(str,aktat->data,aktat->len,1);
      }
  }
  str << endl;

  lp.ende();
  lock();
  lp.schreib(fd);
  unlock();

}

/*---------------------- Paketlog  -------------------------*/
int PaketLogfile::log(Nachricht& msg)
{
  struct timeval tv;
  time_t tsec;
  char zstr[40];
  lp.init();
  ostrstream str(lp.puffer,lp.maxlen());
  
  gettimeofday(&tv,NULL);
  tsec = tv.tv_sec;
  strftime(zstr,40,"%a %d.%m.%Y %H:%M:%S ",localtime(&tsec));
  str << zstr;
  msg.print(str);
  str << endl;

  lp.ende();
  lock();
  lp.schreib(fd);
  unlock();
  
}

int parsepaketloglevel(PaketLoglevel& pll,char *level)
{
  if(strncmp(level,"ALL",3)==0)
  {
    pll=PLL_ALL;
  }
  else if(strncmp(level,"AUTH",4)==0)
  {
    pll=PLL_AUTH;
  }
  else if(strncmp(level,"ADMIN",5)==0)
  {
    pll=PLL_ADMIN;
  }
  else if(strncmp(level,"WRONGPW",7)==0)
  {
    pll=PLL_WRONGPW;
  }
  else
  {
    pll=PLL_NONE;
    return -1;
  }
  return 0;
}

#ifdef LOGTEST
Attrdict dict;
char *progname;
int debugflags=7;
char buffer[1024];
Zusenden msg(buffer,1024);

main(int argc,char **argv)
{
  Logspez logsp;
  Logfile *log;
  
  int id = 1;
  u_char nullvector[AUTH_VECTOR_LEN];
  memset((char *)nullvector,'\0',AUTH_VECTOR_LEN);
  
  progname =argv[0];
  
  dict.init(RADDICT);
  if(argv[1]==NULL)
  {
    log=new PaketLogfile;
    log->init("logfile");
  }
  else
  {
    FieldLogfile *flog = new FieldLogfile;
    logsp.init(argv[1]);    
    flog->init(logsp,"logfile");
    log=flog;
  }
  log->begin();
  char *username = "heinzi";
    
  msg.initsend(PW_ACCOUNTING_REQUEST,id,nullvector);
  msg.addattr(PW_USER_NAME,username,strlen(username));
  msg.addattr(dict[PW_ACCT_INPUT_PACKETS],"7");
  msg.addattr(dict[PW_ACCT_OUTPUT_PACKETS],"5");
  msg.addattr(dict[PW_ACCT_STATUS_TYPE],"Stop");
  msg.addattr(dict[PW_ACCT_SESSION_TIME],"10");
  msg.addattr(dict["Acct-Input-Octets"],"9");
  msg.addattr(dict["Acct-Output-Octets"],"6");
  
  msg.print(cout);
  log->log(msg);
  log->end();  
}
#endif








