/*********************************************************************
 * Modul........: realms.cc                                          *
 * Autor........: Rudolf Weber Informatik Beratung Ravensburg        *
 * Copyright....: 1997 GNU-License                                   *
 *********************************************************************/
#include<stdio.h>
#include<syslog.h>
#include<string.h>
#include"realms.h"
#include"admin_realm.h"
#include"db_realm.h"
#ifdef WITH_CACHING
#include"caching_realm.h"
#endif
#ifdef WITH_ADABAS_REALM
#include"adabas_realm.h"
#endif
#include"logmsg.h"


/*-------------- Realmverwaltung ---------------------------*/
int Realms::init(char *path,char *realmdat)
{
  char puffer[100];
  char realmname[50];
  char realmtyp[50];
  char parameter[50];
  FILE *fp;
  Realm **akt =&wurzel;
  sprintf(datei,"%s/%s",path,realmdat);
  fp=fopen(datei,"r");
  if(fp == NULL)
  {
    logmsg(LOG_ERR,"Kann %s nicht ffnen!",datei);
    exit(1);
  }

  zeile=0;
  while(fgets(puffer,100,fp)!=NULL)
  {
    zeile++;
    if(puffer[0]=='#') continue;
    if(sscanf(puffer,"%s %[^(](%[^)])",realmname,realmtyp,parameter)!=3)
    {
      logmsg(LOG_ERR,"Syntaxerror in %s Zeile %d",datei,zeile);
      continue;
    }
    *akt = openrealm(realmname,realmtyp,parameter);
    if(*akt == NULL) continue;
    akt=&((*akt)->nxt);
  }
  fclose(fp);

  admin = new Admin_realm("admin");
  admin->init("");
}

Realm *Realms::openrealm(const char *name,const char *typ,char *parameter)
{
  Realm *ret=(Realm *)0;
  try
  {
    if(strcmp(typ,"db")==0)
    {
       ret = new Db_realm(name);
       ret->init(parameter);
    }
#ifdef WITH_CACHING
    else if(strcmp(typ,"caching")==0)
    {
      ret = new Caching_realm(name);
      ret->init(parameter);
    }
#endif
#ifdef WITH_ADABAS_REALM
    else if(strcmp(typ,"adabas")==0)
    {
       ret = new Adabas_realm(name);
       ret->init(parameter);
    }
#endif
    else
    {
      throw Syntaxerror("Unbekannter Realmtyp");
    }
  }
  catch(Databaseerror &dberr)
  {
    logmsg(LOG_ERR,"Databaseerror bei Realm %s: %s",name,dberr.what());
    delete ret;
    ret = (Realm *)0;
  }
  catch(Syntaxerror &synerr)
  {
    logmsg(LOG_ERR,"Syntaxerror %s %d: %s\n",datei,zeile,synerr.what());
    delete ret;
    ret = (Realm *)0;
  }
  
  return ret;
}

Realm *Realms::operator[](char *rname)
{
  Realm *akt = wurzel;
  if(rname[0]=='\0') return admin; // wenn kein Realm->adminrealm

  while(akt!=NULL && strcmp(rname,akt->name())!=0)
  {
    akt=akt->nxt;
  }
  return (akt!=NULL ? akt : (Realm *)NULL);

}

int Realms::anfang()
{
  if(admin) admin->anfang();
  for(Realm *a=wurzel;a!=NULL;a=a->nxt)
  {
    try
    {
      a->anfang();
    }
    catch(Databaseerror &e)
    {
      syslog(LOG_ERR,"Realm::anfang %s: %s ",a->name(),e.what());
      return 0;
    }
    catch(Error &e)
    {
      syslog(LOG_ERR,"anfang %s: %s ",a->name(),e.what());
    }
  }
  return 0;
}

int Realms::ende()
{
  if(admin) admin->ende();
  for(Realm *a=wurzel;a!=NULL;a=a->nxt)
  {
    a->ende();
  }
  return 0;
}

#ifdef REALMSTEST
#include"config.h"

Realms realms;

Attrdict dict;
char *progname;
int debugflags = 0;

int main(int argc,char **argv)
{
  logmsginit(LOGMSG_STDERR,argv[0],0,0);
  realms.init(RADUSERSDBPATH,"realms");
}
#endif
