/*****************************************************************************
 * Liste aus Blockfeld
 * Implementiert Doppeltverkettete Liste auf einem Feld von Blcken fester
 * Lnge
 *****************************************************************************/
#ifndef LABFELD_H
#define LABFELD_H
#include<sys/types.h>

typedef unsigned int index_t;
const index_t INDEX_ENDE = ~0;

/* Verkettungsfeld auf der Speicherstruktur*/
struct Labfeld_v
{
  index_t vorg;
  index_t nachf;
};

struct Labfeld_gemein
{
  index_t erstes;
  index_t letztes;
  size_t blockgr;    // die Gre des Blocks ist auch der Abstand zwischen
                     // den Verkettungsfeldern
};

class Labfeld
{
  Labfeld_v *anfang;  // Zeigt auf das Verkettungsfeld im ersten Block
  Labfeld_gemein *g;
public:
  Labfeld(void *bf,size_t pblockgr,Labfeld_gemein *pg) // gesamt
    {
      anfang=(Labfeld_v *)bf;
      g=pg;
      init(pblockgr);
    }
  Labfeld(void *bf,Labfeld_gemein *pg) // wurde schon initialisiert
    {
      anfang=(Labfeld_v *)bf;
      g=pg;
    }
  void init(size_t pblockgr)
    {
      g->blockgr=pblockgr;
      g->erstes=INDEX_ENDE;
      g->letztes=INDEX_ENDE;
    }
  index_t erstes() { return g->erstes; }
  index_t letztes() {return g->letztes; }
  Labfeld_v *ptr(index_t pos) const
    { 
      return (Labfeld_v *)((char *)anfang + g->blockgr*pos);
    }

  void ausketten(index_t pos)
    {
      Labfeld_v *akt = ptr(pos);
      if(akt->vorg == INDEX_ENDE) g->erstes=akt->nachf;
      else ptr(akt->vorg)->nachf = akt->nachf;
      if(akt->nachf == INDEX_ENDE) g->letztes=akt->vorg;
      else ptr(akt->nachf)->vorg=akt->vorg;
      akt->vorg = INDEX_ENDE; akt->nachf=INDEX_ENDE;
    }
  void ananfang(index_t pos)
    {
      Labfeld_v *akt = ptr(pos);
      akt->vorg=INDEX_ENDE;
      akt->nachf=g->erstes;
      if(g->erstes == INDEX_ENDE) 
      {
	g->erstes=g->letztes=pos;
      }
      else
      {
	ptr(g->erstes)->vorg = pos;
      }
      g->erstes=pos;
    }
  void ansende(index_t pos)
    {
      Labfeld_v *akt = ptr(pos);
      akt->vorg = g->letztes;
      akt->nachf = INDEX_ENDE;
      if(g->letztes == INDEX_ENDE) 
      {
	g->erstes=g->letztes=pos;
      }
      else
      {
	ptr(g->letztes)->nachf = pos;
      }
      g->letztes = pos;
      
    }
  friend class Labfeld_it;
};

class Labfeld_it
{
  const Labfeld *il;
  index_t ipos;
public:
  Labfeld_it(const Labfeld &l) { il = &l; ipos=il->g->erstes; }
  index_t operator()() { return ipos; }
  index_t next() 
    { 
      return (ipos != INDEX_ENDE ? ipos=(il->ptr(ipos))->nachf : INDEX_ENDE);
    }
};
#endif


