#ifndef MLSVERW_H
#define MLSVERW_H
#include<assert.h>
#include<signal.h>
#include<setjmp.h>
#include"geteiltesfeld.h"

#pragma interface
template<class FE> class MLSverw
{
  GeteiltesFeld<FE,geteilt_verw> feld;
  SemBM bm;
  LS2Labfeld frei;
  LS2Labfeld unbeab;
  LS2Labfeld inarbeit;
  SemSig sig;
  
public:
  MLSverw(key_t k,int psize,int anz,int ml):feld(anz,psize,k),bm(7,k),
    frei(feld.anf(),&(feld.verwdat()->lfg_frei),bm,ml),
    unbeab(feld.anf(),&(feld.verwdat()->lfg_unbeab),bm,ml),
    inarbeit(feld.anf(),&(feld.verwdat()->lfg_inarbeit),bm,ml),
    sig(bm)
  {
    if(feld.soll_vorbelegung())
    {
      frei.init(psize);
      unbeab.init(psize);
      inarbeit.init(psize);

      // Initialisiere die Freiliste
      for(int i=0;i<anz;i++)
      {
	frei.ansende(i);
	feld[i].id=i;
	feld[i].stat = S_frei;
      }
    }
  }
  void clear_anleger() // nach fork im Sohn
    { feld.clear_anleger(); bm.clear_anleger();
    }
  ~MLSverw() {}
  void meister(void *vp);
  void arbeiter();
  void dump();
};

#pragma implementation
const int timeout=5;
template<class FE> void MLSverw<FE>::meister(void *vp)
{
  struct timeval jetzt;
  char puffer[64];
  index_t neu=INDEX_ENDE;
  index_t gef=INDEX_ENDE;
  FE *akt=NULL;

  for(;;)
  {
    if(neu == INDEX_ENDE)
    {
      DEBUGCODE(1,cerr << "Besorge mir einen freies Element ..." << endl;)
      neu = frei.letztes_ausketten();
      akt=&(feld[neu]);
    }
    if(neu == INDEX_ENDE)
    {
      logmsg(LOG_WARNING,"Kein Element mehr in freiliste ...");
      continue; // Was mache ich nur ????
    }

    if(empfange_auftrag(*akt,vp)<0) continue;
         
    DEBUGCODE(1,cerr << "Ist Auftrag schon in ... " << endl;)
    
    gef=INDEX_ENDE;
    // Ist Auftrag schon lange fertig ?
    frei.lese_beg();
    for(Labfeld_it it(frei);it()!=INDEX_ENDE && gef==INDEX_ENDE;it.next())
    {
      if(vglauftragsnr(feld[it()],*akt)) gef=it();
    }
    frei.lese_end();
    if(gef!=INDEX_ENDE && feld[gef].stat == S_fertig)
    {
      fertigerauftrag(*akt,vp);
      continue;
    }
    else gef=INDEX_ENDE;

    gettimeofday(&jetzt,NULL);
    // Auftrag schon in Arbeit ?
    inarbeit.lese_beg();
    for(Labfeld_it it(inarbeit);it()!=INDEX_ENDE;it.next())
    {
      if(vglauftragsnr(feld[it()],*akt)) gef=it();
      if(jetzt.tv_sec - feld[it()].anfzeit.tv_sec > timeout)
      {
	logmsg(LOG_NOTICE,"Arbeiter %d braucht mit %s zu lange ...",
	      feld[it()].apid,schreibe_auftragsnr(puffer,feld[it()]));

	if(kill(feld[it()].apid,SIGUSR1)<0)
	{
	  if(errno == ESRCH)
	  {
	    index_t verst = it();
	    
	    arbeiterverstorben(feld[verst],vp);
	   	    
	    inarbeit.lese_end();
	    inarbeit.ausketten(verst);
	    feld[verst].stat = S_frei;
	    frei.ansende(verst);
	    inarbeit.lese_beg();
	  }
	}
      }
    }
    inarbeit.lese_end();
    if(gef!=INDEX_ENDE)
    {
      DEBUGCODE(1,cerr << "Auftrag " << schreibe_auftragsnr(puffer,*akt)
	   << " ist schon in Arbeit \n";)
      continue;
    }
    // Liegt der Auftrag unbearbeitet herum ?
    unbeab.lese_beg();
    for(Labfeld_it it(unbeab);it()!=INDEX_ENDE && gef==INDEX_ENDE;it.next())
    {
      if(vglauftragsnr(feld[it()],*akt)) gef=it();
    }
    unbeab.lese_end();
    if(gef!=INDEX_ENDE)
    {
      DEBUGCODE(1,cerr << "Auftrag " << schreibe_auftragsnr(puffer,*akt)
	   << " liegt in der Warteliste \n";)
      continue;
    }
    
    DEBUGCODE(1,cerr << "Auftrag neu ... ";)

    unbeab.ananfang(neu);
    neu= INDEX_ENDE;
    
    DEBUGCODE(1,cerr << "Signalisiere an Arbeiter" << endl;)
    sig.erzeuge();
  }
}

static jmp_buf arbeiteranf;

void bummler_handler_USR1(int s)
{
  longjmp(arbeiteranf,1);
}

void set_bummler_handler()
{
  sigset_t mask;
  struct sigaction bummler;
  
  bummler.sa_handler = bummler_handler_USR1;
  bummler.sa_flags   = SA_ONESHOT;
  sigemptyset(&(bummler.sa_mask));
  sigaction(SIGUSR1,&bummler,NULL);

  sigemptyset(&mask);
  sigaddset(&mask,SIGUSR1);
  sigprocmask(SIG_UNBLOCK,&mask,NULL);
}

void clear_bummler_handler()
{
  struct sigaction bummler;
  bummler.sa_handler = SIG_IGN;
  bummler.sa_flags = SA_ONESHOT;
  sigemptyset(&(bummler.sa_mask));
  sigaction(SIGUSR1,&bummler,NULL);
}

template<class FE> void MLSverw<FE>::arbeiter()
{
  static int arbeit;
  char puffer[32];
  int jr=0;

  arbeit = INDEX_ENDE;

  clear_bummler_handler();

  if((jr=setjmp(arbeiteranf))>0)
  { // Signal wurde ausgelst
    clear_bummler_handler();

    if(jr==1)
    {
      logmsg(LOG_NOTICE,"Ich habe an %s gearbeitet,aber man laesst mich nicht ... ",schreibe_auftragsnr(puffer,feld[arbeit]));
    }
    else
    {
      logmsg(LOG_NOTICE,"Fehler bei %s aufgetreten",schreibe_auftragsnr(puffer,feld[arbeit]));
    }

    inarbeit.ausketten(arbeit);
    feld[arbeit].stat=S_frei;
    frei.ansende(arbeit);
  }

  for(;;)
  {
    DEBUGCODE(1,cerr << "warte auf auftrag ... " << endl;)
    sig.verbrauche();
    
    DEBUGCODE(1,cerr << "Hohle auftrag:" << endl;)
    arbeit = unbeab.letztes_ausketten();
    if(arbeit == INDEX_ENDE) continue; // hurks ??
    feld[arbeit].apid = getpid();
    gettimeofday(&(feld[arbeit].anfzeit),NULL);
    inarbeit.ansende(arbeit);
    feld[arbeit].stat = S_lokalarbeit;
    
    set_bummler_handler();

    if(bearbeite_auftrag(feld[arbeit])<0) { longjmp(arbeiteranf,2); }
 
    clear_bummler_handler();
    feld[arbeit].stat = S_fertig;
    inarbeit.ausketten(arbeit);
    frei.ananfang(arbeit);
  }
}

template<class FE> void MLSverw<FE>::dump()
{
  char puffer[64];
  cerr << "Freiliste " << feld.verwdat()->lfg_frei.erstes << ":"
       << feld.verwdat()->lfg_frei.letztes << " "
       << "Arbeitsliste " << feld.verwdat()->lfg_inarbeit.erstes << ":"
       << feld.verwdat()->lfg_inarbeit.letztes << " "
       << "Unbearbeitet " << feld.verwdat()->lfg_unbeab.letztes << ":"
       << feld.verwdat()->lfg_unbeab.letztes << endl;

  for(int i=0;i< feld.getanz();i++)
  {
    cerr << i << " Nr.=" << schreibe_auftragsnr(puffer,feld[i])
	 << " Status: " << feld[i].stat
         << " Liste " << feld[i].verk.vorg << ":" << feld[i].verk.nachf
         << endl;
    
  }
}
#endif



