#include<iostream.h>
#include<assert.h>
#include"semver.h"

#if 0
#define DEBUGAUS(aus) cerr << aus << endl;
#else
#define DEBUGAUS(aus)
#endif

SemBM::SemBM(int panzsem,key_t psemkey)
{
  DEBUGAUS("SemBM::SemBM");
  anleger=0;
  semkey=psemkey;
  anzsem=panzsem;

  if((semid = semget(semkey,anzsem,IPC_CREAT|IPC_EXCL|0660))<0)
  {
    if(errno == EEXIST)
    {
      cerr << "Unser Semaphor " << semkey << " existiert schon ..." << endl;
      semid = semget(semkey,0,0);
    }
    if(semid<0)
    {
      cerr << "Semaphoren knnen nicht angelegt werden: " 
           << strerror(errno) << endl;
      exit(1);
    }
  }
  else
  {
    cerr << "Ich habe Sem-ID " << semid << " angelegt !" << endl;
    anleger=1;
  }

  nfrei=0;
}

SemBM::~SemBM()
{
  union semun s;
  if(anleger)
  {
    semctl(semid,IPC_RMID,0,s);
  }
}

int SemBM::alloc(int n)
{
  int ret;
  assert(nfrei+n<=anzsem);
  ret = nfrei;
  nfrei+=n;

  return ret;
}

/*----------------Leserschreiber problem ------------------*/
Leserschreiber2::Leserschreiber2(SemBM& bm,int pmaxleser)
{
  DEBUGAUS("Leserschreiber2::Leserschreiber2");
  semid  = bm.id();
  semnum = bm.alloc(2);
  maxleser=pmaxleser;
  if(bm.soll_vorbelegung())
  {
    union semun s;
    s.val = pmaxleser;
    semctl(semid,semnum,SETVAL,s); // leser
    
    s.val = 0;
    semctl(semid,semnum+1,SETVAL,s); // schreiber
  }
}

void Leserschreiber2::lesesem()
{
  union semun null;
  null.val=0;

  cerr << "L:" << semctl(semid,semnum,GETVAL,null)
       << " S:" << semctl(semid,semnum+1,GETVAL,null) << endl;
}

void Leserschreiber2::lese_beg()
{
  struct sembuf a[2];
  a[0].sem_num=semnum;a[0].sem_op=-1;a[0].sem_flg=SEM_UNDO;
  a[1].sem_num=semnum+1;a[1].sem_op=0;a[1].sem_flg=0;

  semop(semid,a,2);
}

void Leserschreiber2::lese_end()
{
  struct sembuf a;
  a.sem_num=semnum;a.sem_op=1;a.sem_flg=SEM_UNDO;
  
  semop(semid,&a,1);
}

void Leserschreiber2::schr_beg()
{
  struct sembuf ares;
  struct sembuf aexcl;

  aexcl.sem_num=semnum;aexcl.sem_op=-maxleser;aexcl.sem_flg=SEM_UNDO;
  ares.sem_num=semnum+1;ares.sem_op=1;ares.sem_flg=SEM_UNDO;
    
  semop(semid,&ares,1);
  semop(semid,&aexcl,1);
}

void Leserschreiber2::schr_end()
{
  struct sembuf a[2];
  a[0].sem_num=semnum;a[0].sem_op=maxleser;a[0].sem_flg=SEM_UNDO|IPC_NOWAIT;
  a[1].sem_num=semnum+1;a[1].sem_op=-1;a[1].sem_flg=SEM_UNDO|IPC_NOWAIT;
  
  if(semop(semid,a,2)<0)
  {
    if(errno == EAGAIN)
    {
      cerr << "Hurks, ich soll hngen:";lesesem();
      cerr << "dL=" << a[0].sem_op << "dS=" << a[1].sem_op << endl;
    }
    else
    {
      perror("semop");
    }
  }
}

/*--------------Semaphor-Signal ------------------------*/
SemSig::SemSig(SemBM& bm)
{
  DEBUGAUS("SemSig::SemSig");
  semid  = bm.id();
  semnum = bm.alloc(1);

  if(bm.soll_vorbelegung())
  {
    union semun s;
    s.val = 0;
    semctl(semid,semnum,SETVAL,s);
  }
}

void SemSig::erzeuge()
{
  struct sembuf a;
  a.sem_num=semnum;a.sem_op=1;a.sem_flg=IPC_NOWAIT;
  if(semop(semid,&a,1)<0)
  {
    if(errno == EAGAIN)
    {
      cerr << "Hurks, ich soll hngen:" << lesesem() << endl;
    }
    else
    {
      perror("semop");
    }
  }
}

void SemSig::verbrauche()
{
  struct sembuf a;
  a.sem_num=semnum;a.sem_op=-1;a.sem_flg=0;
  semop(semid,&a,1);
}

int SemSig::lesesem()
{
  union semun null;
  null.val=0;

  return semctl(semid,semnum,GETVAL,null);
}

#ifdef TEST
#include<signal.h>
#include<sys/time.h>
SemBM bm(2,10000);
Leserschreiber2 ls(bm,10);

void handler_TERM(int s)
{
  cerr << "Ich sterbe ... " << endl;
  exit(1);
}

void schreiber();
void leser();

main(int argc,char **argv)
{

  struct sigaction siga;

  siga.sa_handler = handler_TERM;
  sigemptyset(&(siga.sa_mask));
  siga.sa_flags = 0;
  sigaction(SIGTERM,&siga,NULL);
  
  if(argc > 1 && argv[1][0]=='s')
  {
    schreiber();
  }
  else
  {
    leser();
  }
  
}

void leser()
{
  struct timeval tv;
  
  for(;;)
  {
    ls.lese_beg();
    cout << "Leser " << getpid() << endl;
    ls.lese_end();

    tv.tv_sec = 0;
    tv.tv_usec = 500000;
    select(0,NULL,NULL,NULL,&tv);
  }
}

void schreiber()
{
  char puffer[64];
  char spuffer[64];

  for(;;)
  {
    cout << "Was soll ich schreiben ?";
    cin >> puffer;

    ls.schr_beg();
    cout << "Bitte Return ...";cin >> puffer;
    ls.schr_end();
  }
}
#endif
