/**************************************************************************
 * Modul...: nachricht.cc                                                 *
 * Zweck...: Implementiert Klasse Nachricht und Unterklassen              *
 *------------------------------------------------------------------------*
 * copyright: 1996,1997 Rudolf Weber Informatikberatung                   *
 *                 Ravensburg,Germany                                     *
 * email: rw@w-4.de                                                       *
 * Rights: see GNU-Copyleft (Distribution only with Source-Code)          *
 * Comericial Users are ask to send money                                 *
 **************************************************************************/
#include<stdlib.h>
#include<time.h>
#include"logmsg.h"
#include"md5.h"
#include"nachricht.h"
#include"ipaddrtools.h"
extern int debugflags;

/*----------------- Msg-code-------------*/
struct mcstr
{
  u_char code;
  char *str;
};

static mcstr mcstrtab[]=
{
  { PW_AUTHENTICATION_REQUEST , "AUTHENTICATION_REQUEST"}, //1
  { PW_AUTHENTICATION_ACK , "AUTHENTICATION_ACK"}, //2
  { PW_AUTHENTICATION_REJECT , "AUTHENTICATION_REJECT"}, //3
  { PW_ACCOUNTING_REQUEST , "ACCOUNTING_REQUEST"}, //4
  { PW_ACCOUNTING_RESPONSE , "ACCOUNTING_RESPONSE"}, //5
  { PW_ACCOUNTING_STATUS , "ACCOUNTING_STATUS"}, //6
  { PW_PASSWORD_REQUEST , "PASSWORD_REQUEST"}, //7
  { PW_PASSWORD_ACK , "PASSWORD_ACK"}, //8
  { PW_PASSWORD_REJECT , "PASSWORD_REJECT"}, //9
  { PW_ACCOUNTING_MESSAGE , "ACCOUNTING_MESSAGE"}, //10
  { PW_ACCESS_CHALLENGE , "ACCESS_CHALLENGE"}, //11
  { 12, NULL },
  { 13, NULL },
  { 14, NULL },
  { 15, NULL },
  { 16, NULL },
  { 17, NULL },
  { 18, NULL },
  { 19, NULL },
  { 20, NULL },
  { 21, NULL },
  { 22, NULL },
  { 23, NULL },
  { 24, NULL },
  { 25, NULL },
  { 26, NULL },
  { 27, NULL },
  { 28, NULL },
  { PW_NEXT_PASSCODE , "NEXT_PASSCODE"}, //29
  { PW_NEW_PIN , "NEW_PIN"}, //30
  { PW_TERMINATE_SESSION , "TERMINATE_SESSION"}, //31
  { PW_PASSWORD_EXPIRED , "PASSWORD_EXPIRED"}, //32
  { 33, NULL },
  { 34, NULL },
  { 35, NULL },
  { 36, NULL },
  { 37, NULL },
  { 38, NULL },
  { 39, NULL },
  { PW_ASCEND_DISCONNECT_REQUEST , "ASCEND_DISCONNECT_REQUEST"}, //40
  { PW_ASCEND_DISCONNECT_ACK , "ASCEND_DISCONNECT_ACK"}, //41
  { PW_ASCEND_DISCONNECT_NAK , "ASCEND_DISCONNECT_NAK"}, //42
  { PW_ASCEND_CHG_FILTERS_REQUEST , "ASCEND_CHG_FILTERS_REQUEST"}, //43
  { PW_ASCEND_CHG_FILTERS_ACK , "ASCEND_CHG_FILTERS_ACK"}, //44
  { PW_ASCEND_CHG_FILTERS_NAK , "ASCEND_CHG_FILTERS_NAK"}, //45
  { PW_RW_NEWUSER_REQUEST , "RW_NEWUSER_REQUEST"}, //46
  { PW_RW_NEWUSER_ACT , "RW_NEWUSER_ACT"}, //47
  { PW_RW_NEWUSER_REJECT , "RW_NEWUSER_REJECT"}, //48
  { PW_RW_USERUPD_REQUEST , "RW_USERUPD_REQUEST"}, //49
  { PW_RW_USERQUERY_REQUEST , "RW_USERQUERY_REQUEST"}, //50
  { PW_RW_USERQUERY_ACK , "RW_USERQUERY_ACK"}, //51
  { PW_RW_USERQUERY_NAK , "RW_USERQUERY_NAK"}, //52
  { PW_RW_LISTUSER_REQUEST , "RW_LISTUSER_REQUEST"} //53
};  
  

const char *msgtype2str(u_char code)
{
  int proxy=0;
  const char *ret;
  int i;
  i= ((code & PW_RW_PROXY_BIT) ? (int)(code & ~PW_RW_PROXY_BIT) : (int)(code))-1;
  
  if(i<sizeof(mcstrtab)/sizeof(struct mcstr))
  {
    return mcstrtab[i].str;
  }
  else return (const char *)NULL;
}

const u_char str2msgtype(const char *str)
{
  mcstr *a;

  for(a=mcstrtab;a<=mcstrtab+sizeof(mcstrtab)/sizeof(struct mcstr);a++)
  {
    if(a->str != NULL && strcmp(str,a->str)==0) return a->code;
  }
  return 0;
}

void listmsgtypes(ostream &ostr)
{
  mcstr *a;

  for(a=mcstrtab;a<=mcstrtab+sizeof(mcstrtab)/sizeof(struct mcstr);a++)
  {
    if(a->str != NULL) ostr << a->str << endl;
  }
}

/*--------------- nachricht -------------------*/
void Nachricht::c_init()
{
  ptr_end = (u_char *)auth+totallen();
  ptr=(u_char *)(auth->data);
}

void Nachricht::print(ostream &ostr)
{
    auth_attr *akt;
    const char *codestr;
    ostr << id() << " " << ((codestr = msgtype2str(code())) ? codestr : "")
         << '\n';
    if(debugflags & 4)
    {
      ostr << "Vektor in hex: ";
      printvectorhex(ostr,auth->vector,AUTH_VECTOR_LEN);
      ostr << "\nAttribute:\n";
    }
    for(akt = first();akt !=NULL && akt->len != 0;akt = next())
    {
	if(akt->id != PW_RW_USER_RECORD ) 
        {
	  akt->print(ostr);
	  ostr << '\n';
	}
    }
    if(akt != NULL && akt->len == 0)
    {
      ostr << "Attribut hat Laenge 0 \n";
    }
}

auth_attr *Nachricht::finde(u_char attribut)
{
  if(ptr==NULL || ptr < (u_char *)(auth->data) || ptr >= ptr_end 
     || ((auth_attr *)ptr)->len==0 )
  {  ptr=(u_char *)(auth->data);
  }
  u_char *anf = ptr;
  int i=0;
  if(ptr_end == (u_char *)(auth->data)) return NULL;
  
  //Zyklisches durchsuchen aller Attribute
  while(((auth_attr *)ptr)->id != attribut && (ptr!=anf||i==0))
  {
    ptr += ((auth_attr *)ptr)->len;
    if(ptr >= ptr_end || ((auth_attr *)ptr)->len == 0 )
    {
	ptr=(u_char *)(auth->data);
    }
    i++;
  }
  return (((auth_attr *)ptr)->id == attribut ? (auth_attr *)ptr : (auth_attr *)NULL);
}

int Nachricht::calc_digest(const u_char *secret)
{
  u_char buffer[512];

  int	secretlen;
      
  memset(buffer, 0, sizeof(buffer));
  /* Use the secret to setup the decryption digest */
  secretlen = strlen((char *)(secret));
  strcpy((char *)buffer, (char *)(secret));
  memcpy((char *)(buffer + secretlen), (char *)(auth->vector),AUTH_VECTOR_LEN);
  md5_calc((u_char *)digest, buffer, secretlen + AUTH_VECTOR_LEN);
  memset(buffer, 0, sizeof(buffer));

  return(0);
}	

/*-------------- Klasse Empfangen ------------------*/


int Empfangen::recv(int sockfd)
{
  int r;
  int salen = sizeof(struct sockaddr_in);
  r = recvfrom(sockfd,(char *)auth,bufflen,0,(struct sockaddr *)absender,&salen);
  if(r < AUTH_HDR_LEN || r != totallen()) return -1;
  //  ptr_end = (u_char *)auth+totallen();
  //  ptr=(u_char *)(auth->data);
  c_init();
  return 0;
}

int Empfangen::init(struct sockaddr_in &ab,u_char *buffer,int length)
{
    absender=&ab;

    auth = (auth_hdr *)buffer;
    if(length =! totallen()) return -1; // Diese Suppe esse ich nicht !
    c_init();
    return 0;
}

void Empfangen::print(ostream &ostr)
{
  cout << "Von " << *absender << " ";
  Nachricht::print(ostr);
}

/* Verify the reply digest */
int Empfangen::check_reply(const u_char altvector[],const u_char *secret)
{
  int     secretlen;
  u_char  calc_digest[AUTH_VECTOR_LEN];
  u_char  reply_digest[AUTH_VECTOR_LEN];
  int ret;


  memcpy ((char *) reply_digest, (char *) auth->vector, AUTH_VECTOR_LEN);
  memcpy ((char *) auth->vector, (char *) altvector, AUTH_VECTOR_LEN);
  secretlen = strlen((char *)secret);
  memcpy ((char *)auth + totallen(), secret, secretlen);
  md5_calc (calc_digest, (u_char *) auth, totallen() + secretlen);
  
  ret = (memcmp ((char *) reply_digest, (char *) calc_digest,
		   AUTH_VECTOR_LEN) != 0 ? -1 : 0);
                                      // restauriere Vector aus replydigest 
  memcpy((char *)(auth->vector),reply_digest,AUTH_VECTOR_LEN);
  memset((char *)auth + totallen(),'\0', secretlen);
  return ret;
}

int Empfangen::abssecret(const char *realm,Secinf *si)
{
  const u_char *csecret = si->secret(absender->sin_addr,realm);

  if(csecret == NULL)
  {
    char ipstr[16];
    logmsg(LOG_AUTH,"Verbotener Zugriff Host %s Realm %s",
	  iptostr(ipstr,absender->sin_addr),realm);
    memset(secret, 0, sizeof(secret));
    return -1;
  }
  strcpy((char *)secret,(const char *)csecret);
  return 0;
}

int Empfangen::bestimme_digest(Secinf *si,const char *realm)
{
  if(abssecret(realm,si)<0) return -1;
  return calc_digest(secret);
}

int Empfangen::authenticate(Secinf *si,const char *realm)
{
  u_char nullv[AUTH_VECTOR_LEN];
  memset(nullv,'\0',AUTH_VECTOR_LEN);

  if(abssecret(realm,si)<0) return -1;
  return check_reply(nullv,secret);   
}

int ciffrepassword(u_char *clearpass,u_char *encryptpass,int len,const u_char *digest)
{
   assert(len <= AUTH_PASS_LEN);
   memcpy((char *)clearpass,(char *)encryptpass, len);
   for(int i=0;i<len;i++)
   {
       clearpass[i] ^= digest[i%AUTH_VECTOR_LEN];
   }
   clearpass[len]='\0';
   
   return 0;
}



/*------------ Klasse Zusenden --------------*/
int Zusenden::initsend(u_char code,u_char id,const u_char *vector)
{
    auth->code = code;
    auth->id = id;
    auth->length = htons(AUTH_HDR_LEN);
    memcpy((char*)(auth->vector),(char *)vector,AUTH_VECTOR_LEN);
    ptr_end = auth->data;
}

void Zusenden::setvector(const u_char *vector)
{
  memcpy((char*)(auth->vector),(char *)vector,AUTH_VECTOR_LEN);
}

int Zusenden::copyreplyattr(auth_attr *ra,int lang)
{
  u_short l = ntohs(auth->length)+lang;
  memcpy((char *)ptr_end,(char *)ra,lang);
  ptr_end += lang;
  auth->length=htons(l);
  
  return 0;
}

/* Calculate the response digest */
int Zusenden::calc_responsedigest(const u_char *rsecret)
{
  u_char digest[AUTH_VECTOR_LEN];
#if DEBUG
  if(debugflags&4)
  { 
    cout << "\ncalc_responsedigest:vorheriger Vektor in hex: ";
    printvectorhex(cout,auth->vector,AUTH_VECTOR_LEN);
    cout << endl;
  }
#endif
  int  secretlen = strlen((char *)rsecret);
  memcpy((char *)auth + totallen(),rsecret, secretlen);
  md5_calc(digest, (u_char *)auth, totallen() + secretlen);
  memcpy((char *)auth->vector, (char *)digest, AUTH_VECTOR_LEN);
  memset((char *)auth + totallen(), '\0', secretlen);
  return 0;
}

/*************************************************************************
*
*	random_vector - Generates a random vector of AUTH_VECTOR_LEN octets.
*
*	Returns:	the vector (call by reference)
*
**************************************************************************/

void random_vector(u_char *vector)
{
  int             randno;
  
  assert(AUTH_VECTOR_LEN % sizeof(int) == 0);
  srand (time (0));
  for(u_char *p  = vector;p < vector+AUTH_VECTOR_LEN;p+=sizeof (int))
  {
    randno = rand ();
    memcpy ((char *) p, (char *) &randno, sizeof (int));
  }
  return;
} /* end of random_vector () */

int Zusenden::send(int sockfd,const struct sockaddr_in& saremote)
{
  int zusenden = totallen();
  int salen = sizeof(saremote);
  
  if(debugflags &0x8)
  {
    if(emplength()!=totallen())
    {
      cerr << "emplength="<<emplength() << " totallen()="<<totallen() << endl;
    }
  }  
  return (sendto(sockfd,(char *)auth,zusenden,0,(struct sockaddr *)&saremote,salen)==zusenden ? 0 : -1);
}
