/*
 *
 *	RADIUS
 *	Remote Authentication Dial In User Service
 *
 * ASCEND: @(#)radius.h	1.3 (95/07/25 00:55:33)
 *
 *
 *	Livingston Enterprises, Inc.
 *	6920 Koll Center Parkway
 *	Pleasanton, CA   94566
 *
 *	Copyright 1992 Livingston Enterprises, Inc.
 *
 *	Permission to use, copy, modify, and distribute this software for any
 *	purpose and without fee is hereby granted, provided that this
 *	copyright and permission notice appear on all copies and supporting
 *	documentation, the name of Livingston Enterprises, Inc. not be used
 *	in advertising or publicity pertaining to distribution of the
 *	program without specific prior permission, and notice be given
 *	in supporting documentation that copying and distribution is by
 *	permission of Livingston Enterprises, Inc.   
 *
 *	Livingston Enterprises, Inc. makes no representations about
 *	the suitability of this software for any purpose.  It is
 *	provided "as is" without express or implied warranty.
 *
 * radiuspoto.h
 * enthaelt nun alle Protokolldaten
 * extrahiert aus radius.h von Rudolf Weber, Ravensburg 
 */
#ifndef RADIUSPROTO_H
#define RADIUSPROTO_H
#include<sys/types.h>


const int AUTH_VECTOR_LEN = 16;
const int AUTH_PASS_LEN = 254;
const int AUTH_STRING_LEN = 254;	/* maximum of 254  need 256 ???? */ 

struct auth_hdr
{
  u_char    code;
  u_char    id;
  u_short   length;
  u_char    vector[AUTH_VECTOR_LEN];
  u_char    data[2];
};

    /* types of passwords */
enum PasswordType 
{
	PWD_UNKNOWN	= 1,	/* not yet determined */
	PWD_SPECIAL,		/* triggers for external authentication */
	PWD_NORMAL		/* plain jane */
};

const int AUTH_HDR_LEN = 20;
#define CHAP_VALUE_LENGTH		16


#define PW_TYPE_STRING			0
#define PW_TYPE_INTEGER			1
#define PW_TYPE_IPADDR			2
#define PW_TYPE_DATE			3
#define PW_TYPE_FILTER_BINARY           4

	/* Types */
#define	PW_AUTHENTICATION_REQUEST	1
#define	PW_AUTHENTICATION_ACK		2
#define	PW_AUTHENTICATION_REJECT	3
#define	PW_ACCOUNTING_REQUEST		4
#define	PW_ACCOUNTING_RESPONSE		5
#define	PW_ACCOUNTING_STATUS		6
#define PW_PASSWORD_REQUEST		7	/* Livingston-specific code */
#define PW_PASSWORD_ACK			8	/* Livingston-specific code */
#define PW_PASSWORD_REJECT		9	/* Livingston-specific code */
#define	PW_ACCOUNTING_MESSAGE		10
#define PW_ACCESS_CHALLENGE		11
#define PW_NEXT_PASSCODE		29	/* Ascend-specific code */
#define PW_NEW_PIN			30	/* Ascend-specific code */
#define PW_TERMINATE_SESSION		31	/* Ascend-specific code */
#define PW_PASSWORD_EXPIRED		32	/* Ascend-spcific code */

#define PW_ASCEND_DISCONNECT_REQUEST	40	/* Ascend-specific code */
#define PW_ASCEND_DISCONNECT_ACK	41	/* Ascend-specific code */
#define PW_ASCEND_DISCONNECT_NAK	42	/* Ascend-specific code */
#define PW_ASCEND_CHG_FILTERS_REQUEST	43	/* Ascend-specific code */
#define PW_ASCEND_CHG_FILTERS_ACK	44	/* Ascend-specific code */
#define PW_ASCEND_CHG_FILTERS_NAK	45	/* Ascend-specific code */
#define PW_RW_NEWUSER_REQUEST           46      /* W-4 specific code */
#define PW_RW_NEWUSER_ACT               47      /* W-4 specific code */
#define PW_RW_NEWUSER_REJECT            48      /* W-4 specific code */
#define PW_RW_USERUPD_REQUEST           49
#define PW_RW_USERQUERY_REQUEST         50      /* W-4 specific code */
#define PW_RW_USERQUERY_ACK             51
#define PW_RW_USERQUERY_NAK             52
#define PW_RW_LISTUSER_REQUEST          53

#define PW_RW_PROXY_BIT                128      /* New */

	/* Attributes */
#define	PW_USER_NAME			1
#define	PW_PASSWORD			2
#define	PW_CHAP_PASSWORD		3
#define	PW_CLIENT_ID			4      /* NAS-IP_Adress */
#define	PW_CLIENT_PORT_ID		5      /* NAS-Port */
#define	PW_USER_SERVICE_TYPE		6
#define	PW_FRAMED_PROTOCOL		7
#define	PW_FRAMED_ADDRESS		8
#define	PW_FRAMED_NETMASK		9
#define	PW_FRAMED_ROUTING		10
#define	PW_FRAMED_FILTER_ID		11
#define	PW_FRAMED_MTU			12
#define	PW_FRAMED_COMPRESSION		13
#define	PW_LOGIN_HOST			14
#define	PW_LOGIN_SERVICE		15
#define	PW_LOGIN_TCP_PORT		16
#define PW_OLD_PASSWORD			17      /* unasigned */
#define PW_PORT_MESSAGE			18
#define PW_DIALBACK_NO			19
#define PW_DIALBACK_NAME		20
#define ASCEND_PW_EXPIRATION		21	/* Officially Un-assigned */
#define PW_FRAMED_ROUTE			22
#define PW_FRAMED_IPXNET		23
#define PW_STATE			24
#define PW_SESSION_TIMEOUT              27      /* integer */
#define PW_IDLE_TIMEOUT                 28      /* integer */
#define PW_TERMINATION_ACTION           29      /* integer */
#define PW_CALLER_ID                    31      /* string  ; von Ascend */
#define PW_CHAP_CHALLENGE               60      /* string */

/* Accounting */
#define PW_ACCT_STATUS_TYPE		40
#define PW_ACCT_DELAY_TIME		41
#define PW_ACCT_INPUT_OCTETS		42
#define PW_ACCT_OUTPUT_OCTETS		43
#define PW_ACCT_SESSION_ID		44
#define PW_ACCT_AUTHENTIC		45
#define PW_ACCT_SESSION_TIME		46
#define PW_ACCT_INPUT_PACKETS           47    /* vergessen im original */
#define PW_ACCT_OUTPUT_PACKETS          48
#define PW_RW_USER_RECORD               50
#define PW_RW_CURRENT_TIME              51      /* W-4 specific code */
#define PW_ASCEND_ARADES		181	
#define	ASCEND_TOKEN_IDLE		199
#define	ASCEND_TOKEN_IMMEDIATE		200	
	/* 201 - 203 allocated but not used in this implementation */
#define ASCEND_REQUIRE_AUTH		201
#define ASCEND_NUMBER_SESSIONS		202
#define ASCEND_AUTHEN_ALIAS		203
#define ASCEND_TOKEN_EXPIRY		204	/* internal */
	/* 205, 206 allocated but not used in this implementation */
#define ASCEND_MENU_SELECTOR		205
#define ASCEND_MENU_ITEM		206
#define	ASCEND_PW_WARNTIME		207
#define	ASCEND_PW_LIFETIME		208
        /* ergnzt aus dem dictionary */
#define ASCEND_PW_EXPECT_CALLBACK       149
#define ASCEND_PW_SEND_SECRET           214
#define ASCEND_PW_ASSIGN_IP_POOL        218
#define ASCEND_PW_DIAL_NUMBER           227
#define ASCEND_PW_SEND_AUTH             231
/*
 *	INTEGER TRANSLATIONS
 */

/*	USER TYPES	*/

#define	PW_LOGIN_USER			1
#define	PW_FRAMED_USER			2
#define	PW_DIALBACK_LOGIN_USER		3
#define	PW_DIALBACK_FRAMED_USER		4
#define PW_DIALOUT_FRAMED_USER          5
#define PW_SHELL_USER                   6

/*	FRAMED PROTOCOLS	*/

#define	PW_PPP				1
#define	PW_SLIP				2

/*	FRAMED ROUTING VALUES	*/

#define	PW_NONE				0
#define	PW_BROADCAST			1
#define	PW_LISTEN			2
#define	PW_BROADCAST_LISTEN		3

/*	FRAMED COMPRESSION TYPES	*/

#define	PW_VAN_JACOBSEN_TCP_IP		1

/*	LOGIN SERVICES	*/

#define	PW_TELNET			0
#define	PW_RLOGIN			1
#define	PW_TCP_CLEAR			2
#define	PW_PORTMASTER			3

/*	AUTHENTICATION LEVEL	*/

#define PW_AUTH_NONE			0
#define PW_AUTH_RADIUS			1
#define PW_AUTH_LOCAL			2

/*	STATUS TYPES	*/

#define PW_STATUS_START			1
#define PW_STATUS_STOP			2
#define PW_STATUS_ALIVE			3

/*	Token Immediate Values	*/
#define TOK_IMM_NO			0
#define TOK_IMM_YES			1

/*	Password Aging Attribute	*/
#define PW_EXPIRATION_NAME	"Ascend-PW-Expiration"	/* attribute name */
/*	Password Aging Value Names for Attribute Ascend-Expiration	*/
#define PW_LIFETIME		"Lifetime-In-Days"	/* value name */
#define PW_WARNTIME		"Days-Of-Warning"	/* value name */

/* ASCEND_EXPECT_CALLBACK */
#define PW_EXPECT_CALLBACK_NO          0
#define PW_EXPECT_CALLBACK_YES         1

/* ASCEND_PW_SEND_AUTH */
#define PW_SEND_AUTH_NONE              0
#define PW_SEND_AUTH_PAP               1
#define PW_SEND_AUTH_CHAP              2
#define PW_SEND_AUTH_MSCHAP            3

#endif











































