/****************************************************************
 * Modul...: rcio.cc                                            *
 * Zweck...: Ein-Ausgaberoutinen                                *
 * Copyright..: modifiziert aus Lars fenebergs Radiusclients    *
 *              von Rudolf Weber Ravensburg                     *
 ****************************************************************/
#include<stdio.h>
#include<termios.h>
#include<unistd.h>
#include<fcntl.h>
#include<signal.h>

/*
 * Function: rc_getstr
 *
 * Purpose: Reads in a string from the user (with or witout echo)
 *
 */

char *rc_getstr(char *buf,int len,char *prompt, int do_echo)
{
  int in, out;
  char *p;
  sigset_t newset;
  sigset_t oldset;
  struct termios term_old, term_new;
  int is_term, flags, old_flags;
  char c;
  int flushed = 0;

  in = fileno (stdin);
  out = fileno (stdout);

  sigemptyset (&newset);
  sigaddset (&newset, SIGINT);
  sigaddset (&newset, SIGTSTP);
  sigaddset (&newset, SIGQUIT);

  sigprocmask (SIG_BLOCK, &newset, &oldset);

  if ((is_term = isatty (in)))
  {

    (void) tcgetattr (in, &term_old);
    term_new = term_old;
    if (do_echo)
      term_new.c_lflag |= ECHO;
    else
      term_new.c_lflag &= ~ECHO;

    if (tcsetattr (in, TCSAFLUSH, &term_new) == 0)
      flushed = 1;

  }
  else
  {

    is_term = 0;
    if ((flags = fcntl (in, F_GETFL, 0)) >= 0)
    {
      old_flags = flags;
      flags |= O_NONBLOCK;

      fcntl (in, F_SETFL, flags);

      while (read (in, &c, 1) > 0)
	/* nothing */ ;

      fcntl (in, F_SETFL, old_flags);

      flushed = 1;
    }
  }

  write (out, prompt, strlen (prompt));

  /* well, this looks ugly, but it handles the following end of line
     markers: \r \r\0 \r\n \n \n\r, at least at a second pass */

  p = buf;
  for (;;)
  {
    if (read (in, &c, 1) <= 0)
      return NULL;

    if (!flushed && ((c == '\0') || (c == '\r') || (c == '\n')))
    {
      flushed = 1;
      continue;
    }

    if ((c == '\r') || (c == '\n'))
      break;

    flushed = 1;

    if (p < buf + len)
    {
      if (do_echo && !is_term)
	write (out, &c, 1);
      *p++ = c;
    }
  }

  *p = '\0';

  if (!do_echo || !is_term)
    write (out, "\r\n", 2);

  if (is_term)
    tcsetattr (in, TCSAFLUSH, &term_old);
  else
  {
    if ((flags = fcntl (in, F_GETFL, 0)) >= 0)
    {
      old_flags = flags;
      flags |= O_NONBLOCK;

      fcntl (in, F_SETFL, flags);

      while (read (in, &c, 1) > 0)
	/* nothing */ ;

      fcntl (in, F_SETFL, old_flags);
    }
  }

  sigprocmask (SIG_SETMASK, &oldset, NULL);

  return (buf);
}
